Package: pedmut
Title: Mutation Models for Pedigree Likelihood Computations
Version: 0.9.1
Authors@R: c(
    person("Magnus Dehli", "Vigeland", , "m.d.vigeland@medisin.uio.no", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9134-4962")),
    person("Thore", "Egeland", , "thore.egeland@nmbu.no", role = "ctb",
           comment = c(ORCID = "0000-0002-3465-8885"))
  )
Description: A collection of functions for modelling mutations in
    pedigrees with marker data, as used e.g. in likelihood computations
    with microsatellite data. Implemented models include equal,
    proportional and stepwise models, as well as random models for
    experimental work, and custom models allowing the user to apply any
    valid mutation matrix. Allele lumping is done following the
    lumpability criteria of Kemeny and Snell (1976), ISBN:0387901922.
License: GPL-3
URL: https://github.com/magnusdv/pedmut
Depends: R (>= 4.2.0)
Imports: lpSolve
Suggests: testthat
Encoding: UTF-8
Language: en-GB
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-17 11:45:42 UTC; magnu
Author: Magnus Dehli Vigeland [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9134-4962>),
  Thore Egeland [ctb] (ORCID: <https://orcid.org/0000-0002-3465-8885>)
Maintainer: Magnus Dehli Vigeland <m.d.vigeland@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-17 15:10:09 UTC
Built: R 4.5.2; ; 2026-05-17 15:35:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.matrix,mutationMatrix)
S3method(as.matrix,mutationModel)
S3method(print,mutationMatrix)
S3method(print,mutationModel)
S3method(toString,mutationMatrix)
S3method(toString,mutationModel)
export(adjustRate)
export(alwaysLumpable)
export(findStationary)
export(getParams)
export(isBounded)
export(isLumpable)
export(isMutationMatrix)
export(isMutationModel)
export(isReversible)
export(isStationary)
export(lumpMutSpecial)
export(lumpedMatrix)
export(lumpedModel)
export(makeReversible)
export(makeStationary)
export(mutRate)
export(mutationMatrix)
export(mutationModel)
export(sexEqual)
export(stabilize)
export(stepwiseReversible)
export(validateMutationMatrix)
export(validateMutationModel)
importFrom(lpSolve,lp)
importFrom(stats,runif)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# pedmut

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/pedmut)](https://CRAN.R-project.org/package=pedmut)
[![](https://cranlogs.r-pkg.org/badges/grand-total/pedmut?color=yellow)](https://cran.r-project.org/package=pedmut)
[![](https://cranlogs.r-pkg.org/badges/last-month/pedmut?color=yellow)](https://cran.r-project.org/package=pedmut)
<!-- badges: end -->

## Introduction

The **pedmut** package is part of the
[**pedsuite**](https://magnusdv.github.io/pedsuite/) ecosystem for
pedigree analysis in R. Its aim is to provide a framework for modelling
mutations in pedigree computations.

Although **pedmut** is self-contained, its main purpose is to be
imported by other **pedsuite** packages, like
[pedprobr](https://github.com/magnusdv/pedprobr) (marker probabilities
and pedigree likelihoods), [forrel](https://github.com/magnusdv/forrel)
(forensic pedigree analysis) and
[dvir](https://github.com/magnusdv/dvir).

For the theoretical background of mutation models and their properties
(stationarity, reversibility, lumpability), I recommend Chapter 5 of
[Pedigree analysis in
R](https://shop.elsevier.com/books/pedigree-analysis-in-r/vigeland/978-0-12-824430-2),
and the references therein.

## Installation

``` r
# The easiest way to get `pedmut` is to install the entire `pedsuite`:
install.packages("pedsuite")

# Alternatively, you can install just `pedmut`:
install.packages("pedmut")

# If you need the latest development version, install it from GitHub:
# install.packages("devtools")
devtools::install_github("magnusdv/pedmut")
```

## A simple likelihood example

The examples below require the packages **pedtools** and **pedprobr** in
addition to **pedmut**. The first two are core members of the pedsuite
and can be loaded collectively with `library(pedsuite)`.

``` r
library(pedsuite)
library(pedmut)
```

The figure below shows a father and son who are homozygous for different
alleles. We assume that the locus is an autosomal marker with two
alleles, labelled 1 and 2.

``` r
# Create pedigree
x = nuclearPed(father = "fa", mother = "mo", child = "boy")

# Add marker
x = addMarker(x, fa = "1/1", boy = "2/2")

# Plot with genotypes
plot(x, marker = 1)
```

<img src="man/figures/README-ex1-ped-1.png" style="display: block; margin: auto;" />

The data clearly constitutes a *Mendelian error*, and gives a likelihood
of 0 without mutation modelling:

``` r
likelihood(x)
#> [1] 0
```

The following code sets a simple mutation model and recomputes the
pedigree likelihood.

``` r
x2 = setMutmod(x, model = "equal", rate = 0.1)

likelihood(x2)
#> [1] 0.0125
```

Under the mutation model, the combination of genotypes is no longer
impossible, yielding a non-zero likelihood. To see details about the
mutation model, we can use the `mutmod()` accessor:

``` r
mutmod(x2, marker = 1)
#> Unisex mutation matrix:
#>     1   2
#> 1 0.9 0.1
#> 2 0.1 0.9
#> 
#> Model: Equal 
#> Rate: 0.1 
#> Frequencies: 0.5, 0.5 
#> 
#> Bounded: Yes 
#> Stationary: Yes 
#> Reversible: Yes 
#> Lumpable: Always
```

## Mutation models

A mutation matrix in **pedmut** is a stochastic matrix, with each row
summing to 1, where the rows and columns are named with allele labels.

Two central functions of package are `mutationMatrix()` and
`mutationModel()`. The first constructs a single mutation matrix
according to various model specifications. The second produces what is
typically required in applications, namely a list of *two* mutation
matrices, named “male” and “female”.

The mutation models currently implemented in **pedmut** are:

- `equal`: All mutations equally likely; probability `1-rate` of no
  mutation. Parameters: `rate`.

- `proportional`: Mutation probabilities are proportional to the target
  allele frequencies. Parameters: `rate`, `afreq`.

- `onestep`: Applicable if all alleles are integers. Mutations are
  allowed only to the nearest integer neighbour. Parameters: `rate`.

- `stepwise`: For this model alleles must be integers or single-decimal
  microvariants (e.g. 17.1). Mutation rates depend on group (integer vs
  microvariant), with `rate` for same-group and `rate2` for
  between-group mutations. Mutations also depend on step size; the
  `range` parameter gives the relative probability of mutating n+1 steps
  versus n steps. Parameters: `rate`, `rate2`, `range`.

- `dawid`: A reversible stepwise mutation model, following the approach
  of Dawid et al. (2002). Parameters: `rate`, `range`.

- `random`: Generates a random mutation matrix, optionally conditioned
  on a fixed overall mutation rate. Parameters: `rate`, `seed` (both
  optional).

- `trivial`: Diagonal mutation matrix with 1 on the diagonal.
  Parameters: None.

- `custom`: Any valid mutation matrix provided by the user. Parameters:
  `matrix`.

## Model properties

Several properties of mutation models are of interest (both theoretical
and practical) for likelihood computations. The **pedmut** package
provides utility functions for quickly checking these:

- `isBounded(M, afreq)`: Checks if `M` is bounded by the allele
  frequencies, meaning that the probability of mutating into an allele
  never exceeds the population frequency of that allele. Unbounded
  models may give counter-intuitive results, like LR \> 1 in a paternity
  case where the alleged father and child have no alleles in common.

- `isStationary(M, afreq)`: Checks if `afreq` is a right eigenvector of
  the mutation matrix `M`. Stationary models have the desirable property
  that allele frequencies don’t change across generations.

- `isReversible(M, afreq)`: Checks if `M` together with `afreq` form a
  *reversible* Markov chain, i.e., that they satisfy the [detailed
  balance](https://en.wikipedia.org/wiki/Detailed_balance) criterion.

- `isLumpable(M, lump)`: Checks if `M` allows clustering (“lumping”) of
  a given subset of alleles. This implements the necessary and
  sufficient condition of *strong lumpability* of Kemeny and Snell
  (*Finite Markov Chains*, 1976).

- `alwaysLumpable(M)`: Checks if `M` allows lumping of *any* allele
  subset.

## Further examples

An `equal` model with rate 0.1:

``` r
mutationMatrix("equal", rate = 0.1, alleles = c("a", "b", "c"))
#>      a    b    c
#> a 0.90 0.05 0.05
#> b 0.05 0.90 0.05
#> c 0.05 0.05 0.90
#> 
#> Model: Equal 
#> Rate: 0.1 
#> 
#> Lumpable: Always
```

Next, a `proportional` model with rate 0.1. Note that this model depends
on the allele frequencies.

``` r
mutationMatrix("prop", rate = 0.1, alleles = c("a", "b", "c"), afreq = c(0.7, 0.2, 0.1))
#>            a          b          c
#> a 0.93478261 0.04347826 0.02173913
#> b 0.15217391 0.82608696 0.02173913
#> c 0.15217391 0.04347826 0.80434783
#> 
#> Model: Proportional 
#> Rate: 0.1 
#> Frequencies: 0.7, 0.2, 0.1 
#> 
#> Bounded: Yes 
#> Stationary: Yes 
#> Reversible: Yes 
#> Lumpable: Always
```

To illustrate the `stepwise` model, we recreate the mutation matrix in
Section 2.1.3 of Simonsson and Mostad (FSI:Genetics, 2015). This is done
as follows:

``` r
mutationMatrix(model = "stepwise", alleles = c("16", "17", "18", "16.1", "17.1"),
               rate = 0.003, rate2 = 0.001, range = 0.5)
#>                16           17           18         16.1         17.1
#> 16   0.9960000000 0.0020000000 0.0010000000 0.0005000000 0.0005000000
#> 17   0.0015000000 0.9960000000 0.0015000000 0.0005000000 0.0005000000
#> 18   0.0010000000 0.0020000000 0.9960000000 0.0005000000 0.0005000000
#> 16.1 0.0003333333 0.0003333333 0.0003333333 0.9960000000 0.0030000000
#> 17.1 0.0003333333 0.0003333333 0.0003333333 0.0030000000 0.9960000000
#> 
#> Model: Stepwise 
#> Rate: 0.003 
#> 
#> Lumpable: Not always
```

A simpler version of the `stepwise` model above, is the `onestep` model,
in which only the immediate neighbouring integers are reachable by
mutation. This model is only applicable when all alleles are integers.

``` r
mutationMatrix(model = "onestep", alleles = c("16", "17", "18"), rate = 0.04)
#>      16   17   18
#> 16 0.96 0.04 0.00
#> 17 0.02 0.96 0.02
#> 18 0.00 0.04 0.96
#> 
#> Model: Onestep 
#> Rate: 0.04 
#> 
#> Lumpable: Not always
```

<!-- README.md is generated from README.Rmd. Please edit that file -->

# pedmut

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/pedmut)](https://CRAN.R-project.org/package=pedmut)
[![](https://cranlogs.r-pkg.org/badges/grand-total/pedmut?color=yellow)](https://cran.r-project.org/package=pedmut)
[![](https://cranlogs.r-pkg.org/badges/last-month/pedmut?color=yellow)](https://cran.r-project.org/package=pedmut)
<!-- badges: end -->

## Introduction

The **pedmut** package is part of the
[**pedsuite**](https://magnusdv.github.io/pedsuite/) ecosystem for
pedigree analysis in R. Its aim is to provide a framework for modelling
mutations in pedigree computations.

Although **pedmut** is self-contained, its main purpose is to be
imported by other **pedsuite** packages, like
[pedprobr](https://github.com/magnusdv/pedprobr) (marker probabilities
and pedigree likelihoods), [forrel](https://github.com/magnusdv/forrel)
(forensic pedigree analysis) and
[dvir](https://github.com/magnusdv/dvir).

For the theoretical background of mutation models and their properties
(stationarity, reversibility, lumpability), I recommend Chapter 5 of
[Pedigree analysis in
R](https://shop.elsevier.com/books/pedigree-analysis-in-r/vigeland/978-0-12-824430-2),
and the references therein.

## Installation

``` r
# The easiest way to get `pedmut` is to install the entire `pedsuite`:
install.packages("pedsuite")

# Alternatively, install just `pedmut`:
install.packages("pedmut")

# For the latest development version, install from GitHub:
# install.packages("remotes")
remotes::install_github("magnusdv/pedmut")
```

## A simple likelihood example

The examples below require the packages **pedtools** and **pedprobr** in
addition to **pedmut**. The first two are core members of the pedsuite
and can be loaded collectively with `library(pedsuite)`.

``` r
library(pedsuite)
library(pedmut)
```

The figure below shows a father and son who are homozygous for different
alleles. We assume that the locus is an autosomal marker with two
alleles, labelled 1 and 2.

``` r
# Create pedigree
x = nuclearPed(father = "fa", mother = "mo", child = "boy")

# Add marker
x = addMarker(x, fa = "1/1", boy = "2/2")

# Plot with genotypes
plot(x, marker = 1)
```

<img src="man/figures/README-ex1-ped-1.png" alt="" style="display: block; margin: auto;" />

The data clearly constitutes a *Mendelian error*, and gives a likelihood
of 0 without mutation modelling:

``` r
likelihood(x)
#> [1] 0
```

The following code sets a simple mutation model and recomputes the
pedigree likelihood.

``` r
x2 = setMutmod(x, model = "equal", rate = 0.1)

likelihood(x2)
#> [1] 0.0125
```

Under the mutation model, the combination of genotypes is no longer
impossible, yielding a non-zero likelihood. To see details about the
mutation model, we can use the `mutmod()` accessor:

``` r
mutmod(x2, marker = 1)
#> Unisex mutation matrix:
#>     1   2
#> 1 0.9 0.1
#> 2 0.1 0.9
#> 
#> Model: Equal 
#> Rate: 0.1 
#> Frequencies: 0.5, 0.5 
#> 
#> Bounded: Yes 
#> Stationary: Yes 
#> Reversible: Yes 
#> Lumpable: Always 
#> Overall rate: 0.1
```

## Mutation models

A mutation matrix in **pedmut** is a stochastic matrix, with each row
summing to 1, where the rows and columns are named with allele labels.

Two central functions of package are `mutationMatrix()` and
`mutationModel()`. The first constructs a single mutation matrix
according to various model specifications. The second produces what is
typically required in applications, namely a list of *two* mutation
matrices, named “male” and “female”.

The mutation models currently implemented in **pedmut** are:

- `equal`: All mutations equally likely; probability `1-rate` of no
  mutation. Parameters: `rate`.

- `proportional`: Mutation probabilities are proportional to the target
  allele frequencies. Parameters: `rate`, `afreq`.

- `onestep`: Applicable if all alleles are integers. Mutations are
  allowed only to the nearest integer neighbour. Parameters: `rate`.

- `stepwise`: For this model alleles must be integers or single-decimal
  microvariants (e.g. 17.1). Mutation rates depend on group (integer vs
  microvariant), with `rate` for same-group and `rate2` for
  between-group mutations. Mutations also depend on step size; the
  `range` parameter gives the relative probability of mutating n+1 steps
  versus n steps. Parameters: `rate`, `rate2`, `range`.

- `dawid`: A reversible stepwise mutation model, following the approach
  of Dawid et al. (2002). Parameters: `rate`, `range`.

- `random`: Generates a random mutation matrix, optionally conditioned
  on a fixed overall mutation rate. Parameters: `rate`, `seed` (both
  optional).

- `trivial`: Diagonal mutation matrix with 1 on the diagonal.
  Parameters: None.

- `custom`: Any valid mutation matrix provided by the user. Parameters:
  `matrix`.

## Model properties

Several properties of mutation models are of interest (both theoretical
and practical) for likelihood computations. The **pedmut** package
provides utility functions for quickly checking these:

- `mutRate(M, afreq)`: Computes the overall mutation rate, given by the
  formula `1 - sum p_i m_ii`, where `p_i` is the population frequency of
  allele `i` and `m_ii` is the non-mutation probability for allele `i`.

- `isBounded(M, afreq)`: Checks if `M` is bounded by the allele
  frequencies, meaning that the probability of mutating into an allele
  never exceeds the population frequency of that allele. Unbounded
  models may give counter-intuitive results, like LR \> 1 in a paternity
  case where the alleged father and child have no alleles in common.

- `isStationary(M, afreq)`: Checks if `afreq` is a right eigenvector of
  the mutation matrix `M`. Stationary models have the desirable property
  that allele frequencies don’t change across generations.

- `isReversible(M, afreq)`: Checks if `M` together with `afreq` form a
  *reversible* Markov chain, i.e., that they satisfy the [detailed
  balance](https://en.wikipedia.org/wiki/Detailed_balance) criterion.

- `isLumpable(M, lump)`: Checks if `M` allows clustering (“lumping”) of
  a given subset of alleles. This implements the necessary and
  sufficient condition of *strong lumpability* of Kemeny and Snell
  (*Finite Markov Chains*, 1976).

- `alwaysLumpable(M)`: Checks if `M` allows lumping of *any* allele
  subset.

## Further examples

An `equal` model with rate 0.1:

``` r
mutationMatrix("equal", rate = 0.1, alleles = c("a", "b", "c"))
#>      a    b    c
#> a 0.90 0.05 0.05
#> b 0.05 0.90 0.05
#> c 0.05 0.05 0.90
#> 
#> Model: Equal 
#> Rate: 0.1 
#> 
#> Lumpable: Always 
#> Overall rate: NA
```

Next, a `proportional` model with rate 0.1. Note that this model depends
on the allele frequencies.

``` r
mutationMatrix("prop", rate = 0.1, alleles = c("a", "b", "c"), afreq = c(0.7, 0.2, 0.1))
#>            a          b          c
#> a 0.93478261 0.04347826 0.02173913
#> b 0.15217391 0.82608696 0.02173913
#> c 0.15217391 0.04347826 0.80434783
#> 
#> Model: Proportional 
#> Rate: 0.1 
#> Frequencies: 0.7, 0.2, 0.1 
#> 
#> Bounded: Yes 
#> Stationary: Yes 
#> Reversible: Yes 
#> Lumpable: Always 
#> Overall rate: 0.1
```

To illustrate the `stepwise` model, we recreate the mutation matrix in
Section 2.1.3 of Simonsson and Mostad (FSI:Genetics, 2015). This is done
as follows:

``` r
mutationMatrix(model = "stepwise", alleles = c("16", "17", "18", "16.1", "17.1"),
               rate = 0.003, rate2 = 0.001, range = 0.5)
#>                16           17           18         16.1         17.1
#> 16   0.9960000000 0.0020000000 0.0010000000 0.0005000000 0.0005000000
#> 17   0.0015000000 0.9960000000 0.0015000000 0.0005000000 0.0005000000
#> 18   0.0010000000 0.0020000000 0.9960000000 0.0005000000 0.0005000000
#> 16.1 0.0003333333 0.0003333333 0.0003333333 0.9960000000 0.0030000000
#> 17.1 0.0003333333 0.0003333333 0.0003333333 0.0030000000 0.9960000000
#> 
#> Model: Stepwise 
#> Rate: 0.003 
#> 
#> Lumpable: Not always 
#> Overall rate: NA
```

A simpler version of the `stepwise` model above, is the `onestep` model,
in which only the immediate neighbouring integers are reachable by
mutation. This model is only applicable when all alleles are integers.

``` r
mutationMatrix(model = "onestep", alleles = c("16", "17", "18"), rate = 0.04)
#>      16   17   18
#> 16 0.96 0.04 0.00
#> 17 0.02 0.96 0.02
#> 18 0.00 0.04 0.96
#> 
#> Model: Onestep 
#> Rate: 0.04 
#> 
#> Lumpable: Not always 
#> Overall rate: NA
```

#' Combine alleles in a mutation matrix
#'
#' Reduce a mutation matrix by combining a set of alleles into one *lump*, if
#' this can be done without distorting the mutation process of the remaining
#' alleles. Such allele lumping can give dramatic efficiency improvements in
#' likelihood computations with multi-allelic markers, in cases where only some
#' of the alleles are observed in the pedigree.
#'
#' The lumping implemented in this function is based on the Markov chain lumping
#' theory by Kemeny & Snell (1976). For other, specialised lumping, see
#' [lumpMutSpecial()].
#'
#' @param mutmat A `mutationMatrix` object, typically made with
#'   [mutationMatrix()].
#' @param mutmod A `mutationModel` object, typically made with
#'   [mutationModel()].
#' @param afreq A vector with allele frequencies, of the same length as the size
#'   of `mutmat`. Extracted from the model if not given.
#' @param lump A vector containing the alleles to be lumped together, or a list
#'   of several such vectors.
#' @param check A logical indicating if lumpability (i.e., the row-sum criterium
#'   of Kemeny & Snell) should be checked before lumping. Default: TRUE.
#' @param labelSep A character used to name lumps by pasting allele labels. (For
#'   debugging.)
#'
#' @return A reduced mutation model. If the original matrix has dimensions
#'   \eqn{n\times n}{n*n}, the result will be \eqn{k\times k}{k*k}, where \eqn{k
#'   = n - length(lump) + 1}.
#'
#' @seealso [lumpMutSpecial()].
#'
#' @references Kemeny & Snell (1976). \emph{Finite Markov Chains}. Springer.

#' @examples
#'
#' af = c(.1, .2, .3, .4)
#' names(af) = 1:4
#'
#' ### Example 1: Lumping a mutation matrix
#' mat = mutationMatrix("eq", afreq = af, rate = 0.1)
#' mat
#'
#' # Lump
#' lumpedMatrix(mat, lump = 3:4)
#' lumpedMatrix(mat, lump = 2:4)
#'
#' # Example 2: Full model, proportional
#' mutrate = list(male = 0.1, female = 0.2)
#' mod = mutationModel("prop", afreq = af, rate = mutrate)
#' mod
#'
#' # Lump
#' lumpedModel(mod, lump = 2:4)
#'
#' @export
lumpedMatrix = function(mutmat, lump, afreq = NULL, check = TRUE, labelSep = NULL) {

  if(!inherits(mutmat, "mutationMatrix"))
    stop2(sprintf("Expected the input to be a `mutationMatrix`, but got a: `%s`", class(mutmat)[1]))

  als = colnames(mutmat)
  lump = prepLump(lump, alleles = als, labelSep = labelSep)
  lumpName = names(lump)

  # If exhaustive lumped, return trivial mutationModel
  if(setequal(lump[[1]], als)) {
    newM = matrix(1, ncol=1, nrow=1, dimnames=list("lump", "lump"))
    mod = newMutationMatrix(newM, model = "trivial")
    return(mod)
  }

  if(check && !isLumpable(mutmat, lump))
    stop2("The model is not lumpable for this set of alleles")

  afreq = if(is.null(afreq)) attr(mutmat, 'afreq') else checkAfreq(afreq, alleles = als)
  lumpedFreq = NULL

  if(length(lump) == 1) {
    lump = lump[[1]]
    keep = .mysetdiff(als, lump)
    newAls = c(keep, lumpName)
    N = length(keep) + 1

    newM = matrix(0, N, N, dimnames = list(newAls, newAls))
    newM[, -N] = mutmat[c(keep, lump[1]), keep]
    newM[, N] = 1 - .rowSums(newM, N, N)

    if(!is.null(afreq))
      lumpedFreq = c(afreq[keep], lump = sum(afreq[lump]))
  }
  else {
    # Multiple nontrivial lumps!

    singles = .mysetdiff(als, unlist(lump, use.names = FALSE))
    lmp1 = vapply(lump, function(lmp) lmp[1], character(1))
    newAls = c(singles, lmp1)
    N = length(newAls)

    singleCols = mutmat[newAls, singles, drop = FALSE]
    lumpedCols = vapply(lump, function(lmp)
      .rowSums(mutmat[newAls, lmp, drop = FALSE], N, length(lmp)), FUN.VALUE = numeric(N))

    newM = cbind(singleCols, lumpedCols)
    rownames(newM) = colnames(newM)

    if(!is.null(afreq)) {
      lmpFrqs = vapply(lump, function(lmp) sum(afreq[match(lmp, als)]), numeric(1))
      lumpedFreq = c(afreq[match(singles, als)], lmpFrqs)
      names(lumpedFreq) = colnames(newM)
    }
  }

  newMutationMatrix(newM, afreq = lumpedFreq, lumpedAlleles = lump,
                   model = attr(mutmat, 'model'), rate = attr(mutmat, 'rate'),
                   seed = attr(mutmat, 'seed'))
}


#' @rdname lumpedMatrix
#' @export
lumpedModel = function(mutmod, lump, afreq = attr(mutmod, "afreq"), check = TRUE) {
  mapFullModel(mutmod, lumpedMatrix, lump = lump, afreq = afreq, check = check)
}


prepLump = function(lump, alleles = NULL, labelSep = NULL) {
  if(!is.list(lump))
    lump = list(lump)

  # Remove empty and trivial lumps
  lump[lengths(lump) < 2] = NULL

  if(!length(lump))
    return(lump)

  # Convert each lump to character vector
  lump = lapply(lump, as.character)

  n = length(lump)
  unl = unlist(lump, use.names = FALSE)

  # Checks
  if(dup <- anyDuplicated(unl))
    stop2("Duplicated entry in `lump`: ", unl[dup])

  if(!is.null(alleles) && !all(unl %in% alleles))
    stop2("Alleles not found in mutation matrix: ", .mysetdiff(unl, alleles))

  # Name lumps
  if(!is.null(labelSep))
    names(lump) = vapply(lump, function(v) paste(v, collapse = labelSep), character(1))

  if(is.null(names(lump))) {
    nms = if(n == 1) "lump" else paste0("lump", seq_len(n))
    if(any(nms %in% alleles))
      nms = make.unique(c(alleles, nms), sep = "")[-seq_along(alleles)]
    names(lump) = nms
  }

  lump
}

test_that("lumping works for custum matrix", {
  m1 = mutationMatrix("custom", matrix = matrix(1/3, ncol=3, nrow=3), alleles=1:3)
  lumped1 = lumpedMatrix(m1, lump = 2:3)
  expect_equal(dim(lumped1), c(2,2))
  expect_equal(colnames(lumped1), c("1", "lump"))
  expect_equal(attr(lumped1, 'lumpedAlleles'), c("2", "3"))
})

test_that("wrong input is caught when lumping matrix", {
  mat = mutationMatrix("eq", alleles = 1:4, rate = 0.1)
  expect_error(lumpedMatrix(mat, lump = 2:3, afreq = 1),
               "Frequency vector does not match the number of alleles")
  expect_error(lumpedMatrix(mat, lump = 2:3, afreq = 1:4),
               "Allele frequencies do not sum to 1")

  mod = mutationModel("eq", alleles = 1:4, rate = 0.1)

  expect_error(lumpedModel(mod, lump = 2:3, afreq = 1),
               "Frequency vector does not match the number of alleles")
  expect_error(lumpedModel(mod, lump = 2:3, afreq = 1:4),
               "Allele frequencies do not sum to 1")
})

#' Transformations to reversibility
#'
#' This function implements three methods for transforming a mutation model
#' `(M,p)` into a reversible one, `(R,p)`. All methods are based on Metropolis-
#' Hastings proposal functions.
#'
#' These transformations may also be applied through the `transform` argument of
#' [mutationMatrix()] and [mutationModel()].
#'
#' @param mutmat A [mutationMatrix()] or [mutationModel()].
#' @param method A character indicating which transformation to use. Either "BA"
#'   (Barker), "MH" (Metropolis-Hastings) or "PR" (preserved rate).
#' @param afreq A vector of allele frequencies. Extracted from `mutmat` if not
#'   provided.
#' @param adjust Logical. If TRUE (default), the overall mutation rate is
#'   adjusted to preserve the original rate; see [adjustRate()]. Not relevant
#'   for method "PR", which by construction always preserves the overall rate.
#'
#' @returns A reversible mutation matrix with the same allele frequencies.
#'
#' @examples
#' m = mutationMatrix("equal", afreq = c(a=0.2, b=0.3, c=0.5), rate = 0.2)
#' makeReversible(m, "BA")
#' makeReversible(m, "MH")
#' makeReversible(m, "PR")
#'
#' makeReversible(m, "BA", adjust = FALSE)  # rate differs!
#'
#' # Apply to full model with different female/male rates
#' mod = mutationModel("equal", afreq = c(a=0.2, b=0.3, c=0.5),
#'                     rate = list(female = 0.1, male = 0.2))
#' modR = makeReversible(mod)
#'
#' @export
makeReversible = function(mutmat, method = c("BA", "MH", "PR"), adjust = TRUE,
                          afreq = NULL) {

  if(isMutationModel(mutmat))
    return(mapFullModel(mutmat, makeReversible, method = method, adjust = adjust, afreq = afreq))

  method = match.arg(method)
  afreq = afreq %||% attr(mutmat, "afreq") %||% stop2("`afreq` must be provided")

  M = as.matrix(mutmat)
  pm = M * afreq
  pmT = t.default(pm)

  # Transform the matrix
  switch(method,
    MH = {
      R = M * pmin(1, pmT/pm)
      R[M == 0] = 0
    },
    BA = {
      R = M * pmT/(pm + pmT)
      R[M == 0] = 0
    },
    PR = {
      # Check if the model is well-defined (formula from paper)
      if(any(colSums(pm) > afreq * (1 + 2*diag(M))))
        stop2("The PR transformation is not well-defined for this model")
      R = (pm + pmT)/(2*afreq)
    }
  )

  # Modify diagonal so that row sums are 1
  diag(R) = 0
  diag(R) = 1 - rowSums(R)

  # Original overall rate
  rate = mutRate(M, afreq)

  # Adjust rate of R if needed
  if(adjust && method != "PR") {
    R = adjustRate(R, newrate = rate, afreq = afreq)
    newrate = rate
  }
  else {
    newrate = mutRate(R, afreq)
  }

  # Return as mutation matrix
  newMutationMatrix(R, "custom", afreq = afreq, rate = newrate)
}

#' Overall mutation rate
#'
#' Calculate the overall mutation rate at a locus, given a mutation model an a
#' set of allele frequencies.
#'
#' The mutation rate is found by the formula `1 - sum(diag(mutmat) * afreq)`.
#'
#' If `mutmat` is a full [mutationModel()], the rate is calculated separately for
#' the male and female matrices.
#'
#' @param mutmat A [mutationMatrix()] or [mutationModel()].
#' @param afreq A vector of allele frequencies.
#'
#' @return A single number, or (if `mutmat` is a [mutationModel()] and the female
#'   and male rates differ) a list of two numbers, named "female" and "male".
#'
#' @examples
#' m = mutationMatrix("stepwise", alleles = 1:4, afreq = c(.1,.2,.3,.4),
#'                    rate = 0.01, rate2 = 1e-6, range = 0.1)
#' r = mutRate(m)
#'
#' stopifnot(all.equal(r, 0.01))
#' @export
mutRate = function(mutmat, afreq = NULL) {
  if(isMutationModel(mutmat)) {
    r = lapply(mutmat, function(m) mutRate(m, afreq))
    return(if(all.equal(r$male, r$female)) r$male else unlist(r))
  }

  afreq = afreq %||% attr(mutmat, "afreq") %||%
      stop2("Argument `afreq` is missing and not present as model attribute")

  1 - sum(diag(mutmat) * afreq)
}

#' Adjust the overall mutation rate of a model
#'
#' Adjusts the overall mutation rate of a model by scaling the off-diagonal
#' matrix entries.
#'
#' The adjusted matrix is calculated as `a * M + (1-a) * I`, where `M` is the
#' original matrix, `a = newrate/rate`, and `I` is the identity matrix.
#'
#' The maximum allowed value of `newrate` (to avoid negative values in the
#' adjusted matrix) is `rate/(1 - m))`, where `m` is the smallest diagonal
#' element in the original matrix.
#'
#' @param mutmat A mutation matrix with nonzero mutation overall rate.
#' @param newrate The new overall mutation rate.
#' @param afreq The allele frequencies. Extracted from the mutation matrix if
#'   not provided.
#' @param rate The current overall mutation rate. Calculated from the input if
#'   not provided.
#'
#' @seealso [mutRate()]
#'
#' @returns A new mutation matrix with the adjusted rate.
#'
#' @examples
#' m = mutationMatrix("equal", afreq = c(a=0.2, b=0.3, c=0.5), rate = 0.2)
#' m
#' adjustRate(m, 0.4)
#'
#' @export
adjustRate = function(mutmat, newrate, afreq = NULL, rate = NULL) {
  afreq = afreq %||% attr(mutmat, "afreq") %||% stop2("`afreq` not found")
  rate = rate %||% mutRate(mutmat, afreq)
  if(rate < sqrt(.Machine$double.eps))
    stop2("Cannot adjust rate for a model with rate 0")

  M = as.matrix(mutmat)

  # Check if within theoretical limit
  mxr = rate/(1 - min(diag(M)))
  if(newrate > mxr)
     stop2(sprintf("Cannot adjust rate to %g; maximum is %g", newrate, mxr))

  a = newrate / rate
  newM = a * M + (1-a) * diag(length(afreq))
  newMutationMatrix(newM, model = "custom", afreq = afreq)
}

stop2 = function(...) {
  a = lapply(list(...), toString)
  a = append(a, list(call. = FALSE))
  do.call(stop, a)
}

# Test that input is a single positive (or similar) integer.
isCount = function(x, minimum = 1) {
  isTRUE(length(x) == 1 &&
           (is.integer(x) || (is.numeric(x) && x == as.integer(x))) &&
           x >= minimum)
}

# Test that input is a single number, with optional range constraints
isNumber = function(x, minimum = NA, maximum = NA) {
  isTRUE(length(x) == 1 &&
           is.numeric(x) &&
           (is.na(minimum) || x >= minimum) &&
           (is.na(maximum) || x <= maximum))
}


`%||%` = function(x, y) {
  if(is.null(x)) y else x
}

# A safer version of base::sample
safe_sample <- function(x, ...) x[sample.int(length(x), ...)]

# Order numerically if appropriate otherwise lex
smartOrder = function(x) {
  if (!is.numeric(x) && !anyNA(suppressWarnings(as.numeric(x))))
    ord = order(as.numeric(x))
  else
    ord = order(x)
  ord
}

# Fast setdiff
.mysetdiff = function(x, y) unique.default(x[match(x, y, 0L) == 0L])

# Fast intersection. NB: assumes no duplicates!
.myintersect = function(x, y) y[match(x, y, 0L)]

.equalNums = function(v, tol = sqrt(.Machine$double.eps)) {
  all(v == v[1]) || abs(max(v) - min(v)) < tol
}


#' @export
as.matrix.mutationMatrix = function(x, ...) {
  attributes(x) = list(dim = dim(x), dimnames = dimnames(x))
  x
}

#' @export
as.matrix.mutationModel = function(x, ...) {
  if(!sexEqual(x))
    stop2("Male and female matrices are not equal")
  m = x$female
  attributes(m) = list(dim = dim(m), dimnames = dimnames(m))
  m
}

#' Test for mutation matrix/model
#'
#' @param x Any object.
#' @returns TRUE or FALSE
#' @examples
#'
#' mat = mutationMatrix("equal", alleles = 1:2, rate = 0.1)
#' isMutationMatrix(mat)
#'
#' isMutationModel(mat) # FALSE (not a complete model)
#'
#' mod = mutationModel(mat)
#' isMutationModel(mod)
#'
#' @export
isMutationModel = function(x) {
  inherits(x, "mutationModel")
}

#' @rdname isMutationModel
#' @export
isMutationMatrix = function(x) {
  inherits(x, "mutationMatrix")
}

#' Stabilization of mutation matrix
#'
#' NB: REPLACED BY [makeStationary]. Produces a mutation matrix close to the
#' input `mutmat`, for which the given frequency vector is the stationary
#' distribution. Several methods for doing this are described by Simonsson and
#' Mostad (2016); only the "PM" method is included here.
#'
#' This function is based on, and reuses code from, the `stabilize()` method of
#' the Familias R package.
#'
#' @param mutmat A mutation matrix.
#' @param afreq A vector of allele frequencies.
#' @param method Either "DP", "RM" or "PM". Currently only "PM" is implemented.
#' @param details A logical. If TRUE, the complete Familias output is included.
#'
#' @return An object of the same class the input `mutmat`; either a matrix, a
#'   `mutationMatrix` or a `mutationModel`.
#'
#' @author Petter Mostad, Thore Egeland, Ivar Simonsson, Magnus D. Vigeland
#' @references Simonsson, Mostad: Stationary Mutation models. (FSI: Genetics,
#'   2016).
#'
#' @examples
#'
#' afreq = c(`1` = .2, `2` = .3, `3` = .5)
#' m = mutationMatrix("stepwise", afreq = afreq,
#'                    rate = 0.1, rate2 = 0.01, range = 0.1)
#' m
#' stabilize(m)
#'
#'
#' ### Example with full model (i.e., male and female)
#'
#' M = mutationModel("stepwise", alleles = 1:3, afreq = afreq,
#'                    rate = list(male = 0.1, female = 0.2),
#'                    rate2 = 0.01, range = 0.1)
#' M
#' stabilize(M)
#'
#' @export
stabilize = function(mutmat, afreq = NULL, method = "PM", details = FALSE) {

  if(isMutationModel(mutmat)) {
    Pm = stabilize(mutmat$male, afreq = afreq, method = method)
    Pf = stabilize(mutmat$female, afreq = afreq, method = method)
    return(mutationModel(list(male = Pm, female = Pf)))
  }
  else if(isMutationMatrix(mutmat)) {
    if(is.null(afreq))
      afreq = attr(mutmat, "afreq")
    P = stabilize(as.matrix(mutmat), afreq, method)
    res = mutationMatrix("custom", matrix = P, afreq = afreq)
    return(res)
  }
  else if(!is.matrix(mutmat))
    stop2("Argument `mutmat` must be either a `matrix`, a `mutationMatrix()` or a `mutationModel()`: ", class(mutmat)[1])

  ### Variables used in Familias code below
  M = as.matrix(mutmat)
  pe = afreq %||% stop2("Argument `afreq` is missing")
  t = 1
  ###

  R = 1 - sum(diag(M) * pe)
  n = dim(M)[1]

  X = t.default(M) - diag(n)
  X[n, ] = rep(1, n)
  v = solve.default(X, c(rep(0, n - 1), 1))
  d = R * v/((1 - sum(diag(M) * v)) * pe)

  if (any(d * (1 - diag(M)) > t))
    stop2("PM stabilization doesn't exist.")

  P = diag(d) %*% (M - diag(n)) + diag(n)
  rownames(P) = colnames(P)

  if(details) {
    fratio = max(max(P/M), max(M/P))
    minS = min(diag(P))
    return(list(stabilized = P, fratio = fratio, mindiag = minS, error = ""))
  }

  P
}

#' Get model parameters
#'
#' Extract model parameters of a mutation matrix/model.
#'
#' @param mut A [mutationModel()] or [mutationMatrix()].
#' @param params A vector contain some or all of the words "model", "rate",
#'   "range", "rate2", "seed". If NULL (default), all present parameters are
#'   included.
#' @param format A numeric code indicating the wanted output format. See Value.
#' @param sep A separator character used to paste male and female values.
#'   Ignored unless `format = 3`.
#'
#' @returns When `mut` is a `mutationModel`, the output format is dictated by the `format`
#'   option, with the following possibilities:
#'
#'   1. A data frame with 2 rows labelled 'female' and 'male'.
#'
#'   2. A data frame with 1 row and female/male columns suffixed by .F/.M
#'   respectively.
#'
#'   3. A data frame with 1 row, in which female/male values are pasted together
#'   (separated with `sep`) if different.
#'
#'   4. A list version of format 1, suitable for programmatic use with `mutationModel()`.
#'
#'   If `mut` is a `mutationMatrix` the output is always a data frame with 1 row.
#'
#' @examples
#' M = mutationModel("equal", 1:2, rate = list(female = 0.2, male = 0.1))
#' getParams(M)
#' getParams(M, format = 2)
#' getParams(M, format = 3)
#' getParams(M, format = 3, sep = "|")
#'
#' pars = getParams(M, format = 4)
#' pars$alleles = 1:2  # Not part of the parameters, but needed to reconstruct the model.
#' M2 = do.call(mutationModel, pars)
#' stopifnot(identical(M, M2))
#'
#' @export
getParams = function(mut, params = NULL, format = 1, sep = "/") {
  if(isMutationModel(mut)) {

    parlist = lapply(mut, function(mat) getParams(mat, params = params))
    params = names(parlist$female)

    res = switch(format,
            # 1
            do.call(rbind, parlist),
            # 2
            {
              names(parlist$female) = paste0(names(parlist$female), ".F")
              names(parlist$male) = paste0(names(parlist$male), ".M")
              res = cbind(parlist$female, parlist$male)
              # Interleave
              npar = length(params)
              idx = as.numeric(rbind(1:npar, 1:npar + npar))
              res[idx]
            },
            # 3
            {
              vals = lapply(params, function(p) {
                valF = parlist$female[[p]]
                valM = parlist$male[[p]]
                if(identical(valF, valM)) valF else paste(valF, valM, sep = sep)
              })
              names(vals) = params
              as.data.frame(vals)
            },
            # 4
            {
              vals = lapply(params, \(v) lapply(parlist, `[[`, v))
              names(vals) = params
              vals
            }
          )
    return(res)
  }

  attrs = attributes(mut)

  if(is.null(params))
    params = intersect(names(attrs), c("model", "rate", "range", "rate2", "seed"))
  else {
    unkn = setdiff(params, c("model", "rate", "range", "rate2", "seed"))
    if(length(unkn))
      stop2("Unknown param: ", unkn)
  }

  vals = lapply(params, function(p) attrs[[p]] %||% NA)
  names(vals) = params
  as.data.frame(vals)
}

#' Mutation models
#'
#' Constructor for the class `mutationModel`. An object of this class is
#' essentially a list of two mutation matrices, named "female" and "male".
#'
#' @param model Either:
#'   * a `mutationModel` object (returned unchanged after validation)
#'   * a single `mutationMatrix` object (will be applied to both genders)
#'   * a list of two `mutationMatrix` objects, named "female" and "male"
#'   * a single model name (see [mutationMatrix()] for valid options)
#'   * a list of two model names, named "female" and "male"
#'
#' @param alleles A character vector with allele labels; passed on to
#'   [mutationMatrix()].
#' @param afreq A numeric vector of allele frequencies; passed on to
#'   [mutationMatrix()].
#' @param matrix A matrix, or a list of two matrices (named "female" and "male")
#' @param rate A numeric mutation rate, or a list of two (named "female" and
#'   "male")
#' @param rate2 A numeric mutation rate, or a list of two (named "female" and
#'   "male"). Required in the "stepwise" model; see [mutationMatrix()] for
#'   details.
#' @param range A positive number, or a list of two (named "female" and "male").
#'   Required in the "stepwise" model; see [mutationMatrix()] for details.
#' @param seed An integer, or a list of two (named "female" and "male").
#' @param transform Either NULL (default) or one of the strings "MH", "BA",
#'   "PR", "PM". See [mutationMatrix()].
#' @param validate A logical, by default TRUE.
#' @param mutmod A `mutationModel` object.
#'
#' @return An object of class `mutationModel`. This is a list of two
#'   `mutationMatrix` objects, named "female" and "male", and the following
#'   attributes:
#'
#'   * `sexEqual`: TRUE if both genders have identical models.
#'   * `alwaysLumpable`: TRUE if both genders have models that are lumpable for
#'   any allele subset.
#'
#' @examples
#' # "Equal" model, same parameters for both genders
#' M1 = mutationModel("eq", alleles = 1:2, rate = 0.1)
#' M1
#'
#' # Different mutation rates
#' M2 = mutationModel("eq", alleles = 1:2, rate = list(male = 0.1, female = 0.01))
#' M2
#'
#' stopifnot(identical(M1$male, M1$female), identical(M2$male, M1$male))
#'
#' # A custom mutation matrix:
#' mat = matrix(c(0,0,1,1), ncol = 2, dimnames = list(1:2, 1:2))
#' M3 = mutationModel(model = "custom", matrix = mat)
#'
#' # Under the hood arguments are passed to `mutationMatrix()`.
#' # Alternatively, this can be done explicitly in the `model` argument
#' M4 = mutationModel(model = mutationMatrix("custom", matrix = mat))
#'
#' stopifnot(identical(M3, M4))
#'
#' # The latter strategy is needed e.g. in `pedtools::marker()`, which gives the
#' # user access to `model`, but not `matrix`.
#'
#' @export
mutationModel = function(model, alleles = NULL, afreq = NULL, matrix = NULL,
                         rate = NULL, rate2 = NULL, range = NULL, seed = NULL,
                         transform = NULL, validate = TRUE) {

  if(isMutationModel(model)) {
    mod = enforceAlleleOrder(model, alleles)
  }
  else if(is.list(model) &&
          length(model) == 2 &&
          setequal(names(model), c("female", "male")) &&
          all(sapply(model, isMutationMatrix))) {
    mod = list(female = enforceAlleleOrder(model$female, alleles),
               male = enforceAlleleOrder(model$male, alleles))
  }
  else if(isMutationMatrix(model)) {
    model = enforceAlleleOrder(model, alleles)
    mod = list(female = model, male = model)
  }
  else if((is.character(model) && length(model) == 1) || is.list(model) && length(model) == 2) {
    # Build models from parameters
    model = validateSingleInput(model, "character")
    matrix = validateMatrixInput(matrix)
    rate = validateSingleInput(rate, "numeric")
    rate2 = validateSingleInput(rate2, "numeric")
    range = validateSingleInput(range, "numeric")
    seed = validateSingleInput(seed, "numeric")

    female = mutationMatrix(model = model$female, alleles = alleles,
                            afreq = afreq, matrix = matrix$female,
                            rate  = rate$female, rate2 = rate2$female,
                            range = range$female, seed = seed$female,
                            transform = transform)

    male = mutationMatrix(model = model$male, alleles = alleles,
                          afreq = afreq, matrix = matrix$male,
                          rate  = rate$male, rate2 = rate2$male,
                          range = range$male, seed = seed$male,
                          transform = transform)

    mod = list(female = female, male = male)
  }
  else
    stop2("`model` must be either:\n",
          " * an object of class `mutationModel`\n",
          " * an object of class `mutationMatrix`` (applied to both genders)`\n",
          " * a list of two `mutationMatrix` objects, named 'female' and 'male'\n",
          " * a character string (see ?mutationModel for valid options)\n",
          " * a list of two characters, named 'female' and 'male'")

  sexEqual = identical(mod$female, mod$male)
  lumpable = alwaysLumpable(mod$female) && (sexEqual || alwaysLumpable(mod$male))
  afreq = attr(mod$female, "afreq")

  mutmod = structure(mod, afreq = afreq,
                     sexEqual = sexEqual,
                     alwaysLumpable = lumpable,
                     class = "mutationModel")

  if(validate)
    validateMutationModel(mutmod)

  mutmod
}

validateSingleInput = function(x, mode) {
  if(is.null(x))
    return(x)

  if(is.vector(x, mode = mode) && length(x) == 1)
    return(list(female = x, male = x))

  if(is.list(x) && length(x) == 2 && setequal(names(x), c("female", "male")))
    return(x)

  stop2("Argument `", deparse(substitute(x)), '` must be either\n',
        " * a single ", mode, "\n",
        ' * a list of length 2, named "female" and "male"')
}

validateMatrixInput = function(x) {
  if(is.null(x))
    return(x)

  if(is.matrix(x))
    return(list(female = x, male = x))

  if(is.list(x) && length(x) == 2 && setequal(names(x), c("female", "male")))
    return(x)

  stop2("Argument `", deparse(substitute(x)), '` must be either\n',
        " * a single matrix\n",
        ' * a list of length 2 matrices, named "female" and "male"')
}

#' @rdname mutationModel
#' @export
validateMutationModel = function(mutmod, alleles = NULL) {
  if(!is.list(mutmod) || length(mutmod) != 2 || !setequal(names(mutmod), c("male", "female"))) {
    stop2("`mutmod` must be a list with elements 'male' and 'female'")
  }

  male = mutmod$male
  female = mutmod$female

  if(is.null(alleles))
    alleles = colnames(male)

  validateMutationMatrix(male, alleles = alleles)

  sexEq = attr(mutmod, "sexEqual")
  if(is.null(sexEq))
    stop2("Mutation model attribute `sexEqual` is not set")

  if(sexEq && !identical(male, female))
    stop2("Mutation model attribute `sexEqual` is falsely set to TRUE")

  if(!sexEq) {
    validateMutationMatrix(female, alleles = alleles)

    if(!identical(attr(male, 'afreq'), attr(female, 'afreq')))
      stop2("Mutation model attribute `afreq` differs for males and females")
  }

  mutmod
}

#' @export
print.mutationModel = function(x, ...) {
  if(attr(x, "sexEqual")) {
    cat("Unisex mutation matrix:\n")
    print(x$female, ...)
  }
  else {
    cat("Female mutation matrix:\n")
    print(x$female, ...)
    cat("* * * * * * * * * * *\n")
    cat("Male mutation matrix:\n")
    print(x$male, ...)
  }
  invisible(x)
}

#' @export
toString.mutationModel = function(x, ...) {
  if(attr(x, "sexEqual"))
    toString(x$female)
  else
    sprintf("%s (male); %s (female)", toString(x$male), toString(x$female))
}



# Permute the allele order of a model or matrix
enforceAlleleOrder = function(m, alleles) {
  if(is.null(alleles))
    return(m)

  if(isMutationModel(m)) {
    m$male = enforceAlleleOrder(m$male, alleles)
    m$female = enforceAlleleOrder(m$female, alleles)
    return(m)
  }

  if(!isMutationMatrix(m))
    stop2("Expected a mutation matrix, not a: ", class(m))

  nms = dimnames(m)
  alleles = as.character(alleles)

  # If already correct order - return
  if(identical(alleles, nms[[1]]))
    return(m)

  if(!setequal(alleles, nms[[1]]) || !setequal(alleles, nms[[2]]))
    stop2("Alleles differ from names of mutation matrix")

  if(length(alleles) > length(nms[[1]]))
    stop2("Duplicated alleles indicated: ", alleles[duplicated[alleles]])

  # Permute
  new_m = m[alleles, alleles]

  # Keep all attributes of original, except `dimnames` and `afreq`
  attrs = attributes(m)
  attrs$dimnames = list(alleles, alleles)
  if(!is.null(attrs$afreq))
    attrs$afreq = attrs$afreq[alleles]

  attributes(new_m) = attrs

  new_m
}

#' @rdname mutationModel
#' @export
sexEqual = function(mutmod) {
  isTRUE(attr(mutmod, "sexEqual"))
}

# Utility for applying a function to both female/male parts of a full model
mapFullModel = function(mutmod, fn, ...) {

  if(!inherits(mutmod, "mutationModel"))
    stop2(sprintf("Expected the input to be a `mutationModel`, but got a: `%s`", class(mutmod)[1]))

  # Apply fn() to female
  outF    = fn(mutmod$female, ...)
  sexeq   = sexEqual(mutmod)

  # If sexes equal reuse, otherwise apply fn() to male
  outM    = if (sexeq) outF else fn(mutmod$male, ...)
  lumpable = alwaysLumpable(outF) && (sexeq || alwaysLumpable(outM))

  structure(
    list(female = outF, male = outM),
    afreq          = attr(outF, "afreq"),
    sexEqual       = sexeq,
    alwaysLumpable = lumpable,
    class          = "mutationModel"
  )
}

test_that("mutation models are validated", {
  alleles = 1:3
  afr = c(.2,.3,.5)
  rate = 0.1
  M = mutationModel("prop", alleles=alleles, afreq=afr, rate=rate)
  expect_silent(validateMutationModel(M))
})


test_that("male and female models are equal by default", {
  alleles = 1:3
  afr = c(.2,.3,.5)
  rate = 0.1
  m = mutationMatrix("prop", alleles=alleles, afreq=afr, rate=rate)

  M = mutationModel("prop", alleles=alleles, afreq=afr, rate=rate)
  expect_identical(M$female, m)
  expect_identical(M$male, m)
})

test_that("stepwise mutation model splits correctly in male/female", {
  alleles = c(1, 1.5, 2,3)
  M = mutationModel("step", alleles = alleles,
                    rate = list(male=0, female=0.1),
                    rate2 = list(male=0.2, female=0),
                    range = list(male=0.2, female=0.3))
  matM = mutationMatrix("step", alleles = alleles,
                        rate = 0, rate2 = 0.2, range = 0.2)
  matF = mutationMatrix("step", alleles = alleles,
                         rate = 0.1, rate2 = 0, range = 0.3)
  expect_identical(M, mutationModel(list(male=matM, female=matF)))
})

#' Mutation matrix
#'
#' Construct mutation matrices for pedigree likelihood computations.
#'
#' Descriptions of the models:
#'
#' * `custom`: Allows any mutation matrix to be provided by the user, in the
#' `matrix` parameter.
#'
#' * `dawid`: A reversible model for integer-valued markers, proposed by Dawid
#' et al. (2002).
#'
#' * `equal`: All mutations equally likely; probability \eqn{1-rate} of no
#' mutation.
#'
#' * `proportional`: Mutation probabilities are proportional to the target
#' allele frequencies.
#'
#' * `random`: This produces a matrix of random numbers, where each row is
#' normalised so that it sums to 1. If `rate` (and `afreq`) is provided, the
#' mutation matrix is conditional on the overall mutation rate.
#'
#' * `onestep`: A mutation model for markers with integer alleles, allowing
#' mutations only to the nearest neighbours in the allelic ladder. For example,
#' `10` may mutate to either `9` or `11`, unless `10` is the lowest allele, in
#' which case `11` is the only option. This model is not applicable to loci with
#' non-integer microvariants.
#'
#' * `stepwise`: A common model in forensic genetics, allowing different
#' mutation rates between integer alleles (like `9`) and non-integer
#' microvariants (like `9.3`). Mutation rates also depend on step size, as
#' controlled by the `range` parameter.
#'
#' * `trivial`: The identity matrix, implying that no mutations are possible.
#'
#' If `transform` is non-NULL, the indicated transformation is applied to the
#' matrix before returning. Currently, there are 4 available options:
#'
#' * `MH`, `BA`, `PR`: See [makeReversible()]
#' * `PM`: See [makeStationary()]
#'
#' @param model A string: either "custom", "dawid", "equal", "proportional",
#'   "random", "onestep", "stepwise" or "trivial".
#' @param matrix When `model` is "custom", this must be a square matrix with
#'   nonnegative real entries and row sums equal to 1.
#' @param alleles A character vector (or coercible to character) with allele
#'   labels. Required in all models, except "custom" if `matrix` has dimnames.
#' @param afreq A numeric vector of allele frequencies. Required in model
#'   "proportional".
#' @param rate A number between 0 and 1. Required in models "equal",
#'   "proportional", "stepwise" and "onestep".
#' @param seed A single number. Optional parameter in the "random" model, passed
#'   on to `set.seed()`.
#' @param rate2 A number between 0 and 1. The mutation rate between integer
#'   alleles and microvariants. Required in the "stepwise" model.
#' @param range A positive number. The relative probability of mutating n+1
#'   steps versus mutating n steps. Required in the "stepwise" and "dawid"
#'   models. Must be in the interval (0,1) for the "dawid" model.
#' @param transform Either NULL (default) or one of the strings "MH", "BA",
#'   "PR", "PM". See Details.
#' @param validate A logical (default: TRUE) indicating whether to validate
#'   custom models.
#' @param mutmat An object of class `mutationMatrix`.
#'
#' @return An object of class `mutationMatrix`, essentially a square numeric
#'   matrix with various attributes. The matrix has entries in `[0, 1]` and all
#'   rows sum to 1. Both row names and column names are the allele labels.
#'
#' @examples
#' mutationMatrix("equal", alleles = 1:3, rate = 0.05)
#'
#' mutationMatrix("random", afreq = c(a=0.3, b=0.7), rate = 0.05, seed = 1)
#'
#' @importFrom stats runif
#' @export
mutationMatrix = function(model = c("custom", "dawid", "equal", "proportional",
                                    "random", "onestep", "stepwise", "trivial"),
                          matrix = NULL, alleles = NULL, afreq = NULL,
                          rate = NULL, seed = NULL, rate2 = NULL, range = NULL,
                          transform = NULL, validate = TRUE) {

  model = match.arg(model)
  alleles = if(!is.null(alleles)) as.character(alleles) else names(afreq)
  afreq = checkAfreq(afreq, alleles)

  # A few ad hocs for custom models
  if(model == "custom")
    alleles = alleles %||% colnames(matrix)
  else if(!is.null(matrix))
    stop2(sprintf("`matrix` cannot be used with the `%s` model", model))

  mutmat = switch(model,
    custom = .custom(matrix, alleles, validate = validate),
    dawid = .dawid(alleles, afreq, rate, range),
    equal = .equal(alleles, rate),
    proportional = .proportional(alleles, afreq, rate),
    random = .random(alleles, afreq, rate, seed),
    stepwise = .stepwise(alleles, rate, rate2, range),
    onestep = .onestep(alleles, rate),
    trivial = .trivial(alleles)
  )

  res = newMutationMatrix(mutmat, model = model, afreq = afreq, rate = rate,
                          rate2 = rate2, range = range, seed = seed)

  if(!is.null(transform)) {
    if(transform == "PM")
      res = makeStationary(res, afreq = afreq)
    else
      res = makeReversible(res, method = transform, adjust = TRUE, afreq = afreq)
  }

  res
}

newMutationMatrix = function(mutmat, model = "custom", afreq = NULL,
                            rate = NULL, rate2 = NULL, range = NULL,
                            lumpedAlleles = NULL, seed = NULL) {
  if(!is.null(afreq)) {
    stationary = isStationary(mutmat, afreq)
    reversible = isReversible(mutmat, afreq)
  }
  else {
    stationary = reversible = NULL
  }
  structure(mutmat,
            model = model,
            afreq = afreq,
            rate = rate,
            rate2 = rate2,
            range = range,
            stationary = stationary,
            reversible = reversible,
            lumpedAlleles = lumpedAlleles,
            seed = seed,
            class = "mutationMatrix")
}


#' @rdname mutationMatrix
#' @export
validateMutationMatrix = function(mutmat, alleles = NULL) {

  dm = dim(mutmat)
  if(is.null(dm) || dm[1] != dm[2] || !is.numeric(mutmat))
    stop2("Mutation matrix is not a square numeric matrix")

  if(anyNA(mutmat))
    stop2("Mutation matrix contains NA elements")

  nms = dimnames(mutmat)
  if(!identical(nms[[1]], nms[[2]]))
    stop2("Mutation matrix has unequal row and column names")

  als = nms[[1]]
  if(!is.null(alleles) && !identical(als, as.character(alleles)))
    stop2("Dimnames of mutation matrix not consistent with indicated alleles")

  afreq = attr(mutmat, "afreq")
  if(!is.null(afreq)) {
    if(!identical(names(afreq), als)) {
      print(afreq)
      stop2("Attribute `afreq` must be named with the allele labels: ", als)
    }
    if(round(sum(afreq), 3) != 1)
      stop2("Allele frequencies do not sum to 1 (after rounding to 3 decimal places): ",
            round(afreq, 3))
  }

  if(any(mutmat < 0))
    stop2("Negative entries found in mutation matrix: ", mutmat[mutmat < 0])
  if(any(mutmat > 1))
    stop2("Entries exceeding 1 found in mutation matrix: ", mutmat[mutmat > 1])

  rs = round(.rowSums(mutmat, dm[1], dm[1]), 3)
  if (any(rs != 1)) {
    rw = which(rs != 1)[1]
    stop2(sprintf("Row %d doesn't sum to 1 (after rounding to 3 decimals), but to: %s", rw, rs[rw]))
  }

  mutmat
}




#' @export
print.mutationMatrix = function(x, includeMatrix = TRUE, includeAttrs = TRUE,
                                includeProperties = TRUE, ...) {
  if(includeMatrix) {
    print(format(x), quote=FALSE, right=TRUE)
  }

  attrs = attributes(x)
  afreq = attrs$afreq

  if(includeAttrs) {
    if(includeMatrix) cat("\n")

    model = attrs$model
    substr(model, 1, 1) = toupper(substr(model, 1, 1))

    rate = attrs$rate
    rate2 = attrs$rate2
    range = attrs$range
    seed = attrs$seed

    cat("Model:", model, "\n")
    cat("Rate:", if(!is.null(rate)) rate else NA, "\n")
    if(!is.null(afreq))
      cat("Frequencies:", toString(afreq), "\n")
    if(model == "random")
      cat("Seed:", if(!is.null(seed)) seed else NA, "\n")
    if(model == "stepwise"){
      cat("Rate2:", rate2, "\n")
      cat("Range:", range,  "\n")
    }
  }

  if(includeProperties) {
    if(includeAttrs) cat("\n")

    if(!is.null(afreq)) {
      cat("Bounded:", if(isBounded(x, afreq)) "Yes" else "No", "\n")
      cat("Stationary:", if(isStationary(x, afreq)) "Yes" else "No", "\n")
      cat("Reversible:", if(isReversible(x, afreq)) "Yes" else "No", "\n")
    }
    cat("Lumpable:", if(alwaysLumpable(x)) "Always" else "Not always", "\n")
    cat("Overall rate:", if(is.null(afreq)) "NA" else round(mutRate(x, afreq), 5), "\n")
  }
}

#' @export
toString.mutationMatrix = function(x, ...) {
  attrs = attributes(x)
  mod = attrs$model
  param = switch(mod,
                 equal =, proportional =, onestep = paste("rate =", attrs$rate),
                 random = paste("seed =", attrs$seed %||% "NULL"),
                 stepwise = sprintf("rate = %g, rate2 = %g, range = %g",
                                    attrs$rate, attrs$rate2, attrs$range),
                 NULL)
  if(!is.null(param))
    mod = sprintf("%s, %s", mod, param)

  mod
}



# Specific model definitions ----------------------------------------------

.trivial = function(alleles) {
  mutmat = diag(length(alleles))
  dimnames(mutmat) = list(alleles, alleles)
  mutmat
}

.equal = function(alleles, rate) {
  checkNullArg(alleles, "equal")
  checkRate(rate, "equal")

  n = length(alleles)
  if(n == 0)
    stop2("No alleles given")
  if(n == 1)
    return(matrix(1, 1L, 1L, dimnames = list(alleles, alleles)))

  m = rate/(n - 1)
  mutmat = matrix(m, ncol = n, nrow = n, dimnames = list(alleles, alleles))
  diag(mutmat) = 1 - rate
  mutmat
}

.proportional = function(alleles, afreq, rate) {
  checkNullArg(alleles, "proportional")
  checkRate(rate, "proportional")
  afreq = checkAfreq(afreq, alleles, checkNULL = TRUE, model = "proportional")

  # Check if mutation rate is too large
  mxr = (1 - sum(afreq^2)) / (1 - min(afreq))
  if(rate > mxr)
    stop2("Model undefined: max `rate` for the given input is: ", mxr)

  n = length(alleles)
  if(n == 0)
    stop2("No alleles given")
  if(n == 1)
    return(matrix(1, 1L, 1L, dimnames = list(alleles, alleles)))

  a = rate / sum(afreq * (1 - afreq))
  mutmat = (1 - a) * diag(n) + a * rep(afreq, each = n)

  dimnames(mutmat) = list(alleles, alleles)
  mutmat
}

.random = function(alleles, afreq = NULL, rate = NULL, seed = NULL) {

  checkNullArg(alleles, "random")
  if(!is.null(seed))
    set.seed(seed)

  n = length(alleles)
  if(n == 0)
    stop2("No alleles given")
  if(n == 1)
    return(matrix(1, 1L, 1L, dimnames = list(alleles, alleles)))

  # Case 1: Unconditional random matrix -------------------------------------

  if(is.null(rate)) {
    m = runif(n^2, min = 0, max = 1)
    mutmat = matrix(m, ncol = n, nrow = n, dimnames = list(alleles, alleles))
    return(mutmat / rowSums(mutmat))
  }

  # Case 2: Conditional on rate ---------------------------------------------

  checkRate(rate, "random")
  if(is.null(afreq))
    stop2("`afreq` is required when generating `random` model with fixed rate")

  ### Step 1: Diagonal m such that m %*% afreq = 1-rate
  s = 1 - rate

  # Feasible starting point (inside [0,1]^n)
  m0 = rep(s, n)  # sum(m0*afreq) = s*sum(afreq) = s

  # Random vector orthogonal to afreq
  u = runif(n)
  v = u - sum(u * afreq)/sum(afreq*afreq) * afreq

  # Range of k such that m0 + k*v is inside the unit cube: 0 < m0 + k*v < 1
  k1 = (0 - m0) / v
  k2 = (1 - m0) / v
  kMin = max(pmin(k1, k2))
  kMax = min(pmax(k1, k2))
  k = runif(1, kMin, kMax)

  # Diagonal vector
  m = m0 + k*v
  mutmat = diag(m)

  ### Step 2: Fill rows randomly to sum 1
  for(i in 1:n) {
    y = runif(n - 1)
    mutmat[i, -i] = y / sum(y) * (1 - mutmat[i, i])
  }

  dimnames(mutmat) = list(alleles, alleles)
  mutmat
}

.stepwise = function(alleles, rate, rate2, range) {
  checkNullArg(alleles, "stepwise")
  checkRate(rate, "stepwise")
  checkRate2(rate2, "stepwise")
  checkRange(range, max = Inf, "stepwise")
  if(rate + rate2 > 1)
    stop2("The total mutation rate `rate + rate2` must be in [0,1]: ", rate + rate2)

  n = length(alleles)
  if(n == 0)
    stop2("No alleles given")
  if(n == 1)
    return(matrix(1, 1L, 1L, dimnames = list(alleles, alleles)))

  alsNum = checkNumericAlleles(alleles, "stepwise")

  # Bug fix: round these!
  microgroup = round((alsNum - round(alsNum))*10)

  # Initialise matrix
  mutmat = matrix(0, ncol = n, nrow = n, dimnames = list(alleles, alleles))

  for (i in 1:n) {
    microcompats = (microgroup == microgroup[i])
    for (j in 1:n) {
      if (i == j) {
        if (all(microcompats)) mutmat[i,j] = 1 - rate
        else if (sum(microcompats) == 1) mutmat[i,j] = 1 - rate2
        else mutmat[i,j] = 1 - rate - rate2
      } else if (microcompats[j])
        mutmat[i,j] = range^abs(alsNum[i] - alsNum[j])
      else
        mutmat[i,j] = rate2/(n - sum(microcompats))
    }
    microcompats[i] = FALSE
    if (any(microcompats))
      mutmat[i, microcompats] = mutmat[i, microcompats]/sum(mutmat[i, microcompats]) * rate
  }

  mutmat
}

.onestep = function(alleles, rate) {
  alsNum = checkIntegerAlleles(alleles, "onestep")
  checkRate(rate, "onestep")

  # Initialise matrix
  n = length(alleles)
  if(n == 0)
    stop2("No alleles given")
  if(n == 1)
    return(matrix(1, 1L, 1L, dimnames = list(alleles, alleles)))

  mutmat = matrix(0, ncol = n, nrow = n, dimnames = list(alleles, alleles))

  # Loop over rows
  for(i in seq_along(alsNum)) {
    neigh = abs(alsNum - alsNum[i]) == 1 # onestep neighbours
    mutmat[i, i] = if(sum(neigh) > 0) 1 - rate else 1
    mutmat[i, neigh] = rate/sum(neigh) # either rate or rate/2
  }

  mutmat
}

.dawid = function(alleles, afreq, rate, range) {
  alsNum = checkIntegerAlleles(alleles, "dawid")
  afreq = checkAfreq(afreq, alleles, checkNULL = TRUE, model = "dawid")

  checkRate(rate, "dawid")
  checkRange(range, max = 1, "dawid")

  n = length(alleles)
  if(n == 0)
    stop2("No alleles given")
  if(n == 1)
    return(matrix(1, 1L, 1L, dimnames = list(alleles, alleles)))

  a = (1 - range^n)/(1 - range)

  stepsize = outer(1:n, 1:n, function(i,j) abs(i-j))
  R = rate/afreq * (1 - range) / (2*range*(n - a)) * range^stepsize
  diag(R) = 0

  # Diagonal values
  dg = 1 - rowSums(R)

  # If undefined (negative diagonal values), return info on max rate
  if(any(dg < 0)) {
    mxr = 1/max(rowSums(R/rate)) # See formulas in Egeland & Vigeland (2025)
    stop2("Model undefined; max `rate` for the given input is: ", mxr)
  }

  diag(R) = dg
  dimnames(R) = list(alleles, alleles)
  R
}

.custom = function(matrix, alleles, validate = TRUE) {
  checkNullArg(matrix, "custom")

  if(!is.matrix(matrix))
    stop2("Custom matrix must be a matrix, not a ", class(matrix))
  if(typeof(matrix) != "double")
    stop2("Custom matrix must be numeric, not ", typeof(matrix))
  if(nrow(matrix) != ncol(matrix))
    stop2("Custom matrix must be square, not ", nrow(matrix), "*", ncol(matrix))

  nullAls = is.null(alleles)
  if(!nullAls) alleles = as.character(alleles)

  # If both dimnames and `alleles` are given: Check match and use to permute matrix
  if(!is.null(dmn <- dimnames(matrix)) && !nullAls) {
    chck = setequal(alleles, dmn[[1L]]) && setequal(alleles, dmn[[2L]])
    if(!chck)
      stop2("Custom matrix names don't match the `alleles` argument")

    matrix = matrix[alleles, alleles]
  }
  else if(!nullAls) {
    if(length(alleles) != dim(matrix)[1L])
      stop2("Length of `alleles` must equal the dimension of `matrix`")

    dimnames(matrix) = list(alleles, alleles)
  }
  else if(is.null(dmn) && nullAls)
    stop2("When custom matrix lacks names, the argument `alleles` cannot be NULL")

  if(validate) validateMutationMatrix(matrix) else matrix
}



# Utilities for checking parameters-------------------------------------------

checkNullArg = function(arg, model = NULL) {
  if(is.null(arg)) {
    argname = deparse(substitute(arg))
    if(is.null(model))
      msg = sprintf("`%s` cannot be NULL with this model", argname)
    else
      msg = sprintf("`%s` cannot be NULL with the `%s` model", argname, model)

    stop2(msg)
  }
}

checkIntegerAlleles = function(alleles, model) {
  checkNullArg(alleles, model)
  alsNum = suppressWarnings(as.numeric(alleles))
  if(any(is.na(alsNum)) || any(round(alsNum) != alsNum)) {
    msg = sprintf("The `%s` model requires all alleles to be integers", model)
    stop2(msg)
  }
  alsNum
}

checkNumericAlleles = function(alleles, model) {
  alsNum = suppressWarnings(round(as.numeric(alleles), 2))
  if(any(is.na(alsNum)))
    stop2(sprintf("The `%s` model requires all alleles to be numeric", model))
  oneDec = round(alsNum, 1) == alsNum
  if(!all(oneDec))
    stop2("Microvariants must be named as a decimal number with one decimal: ", alleles[!oneDec])
  alsNum
}

checkAfreq = function(afreq, alleles = NULL, len = NULL, checkNULL = FALSE, model = NULL) {
  if(checkNULL)
    checkNullArg(afreq, model)

  if(is.null(afreq))
    return(afreq)

  if(!is.numeric(afreq))
    stop2("Expected frequency vector to be a numeric, not ", class(afreq))

  if(!is.null(len) && length(afreq) != len)
    stop2(sprintf("Expected frequency vector to have length %d, not %d", len, length(afreq)))

  if(round(sum(afreq), 3) != 1)
    stop2("Allele frequencies do not sum to 1: ", round(sum(afreq),3))

  if(!is.null(alleles)) {

    if(length(afreq) != length(alleles))
      stop2(sprintf("Frequency vector does not match the number of alleles (%d)", length(alleles)))

    if(!is.null(nms <- names(afreq))) {
      if(!setequal(nms, alleles))
        stop2("Names of frequency vectors do not match alleles: ", setdiff(nms, alleles))

      # Sort
      afreq = afreq[alleles]
    }
    else
      names(afreq) = alleles
  }

  afreq
}

checkRate = function(rate, model) {
  checkNullArg(rate, model)
  if(!isNumber(rate, minimum = 0, maximum = 1))
    stop2("`rate` must be a number in the interval `[0,1]`: ", rate)
}

checkRate2 = function(rate2, model) {
  checkNullArg(rate2, model)
  if(!isNumber(rate2, minimum = 0, maximum = 1))
    stop2("`rate2` must be a number in the interval `[0,1]`: ", rate2)
}

checkRange = function(range, max = Inf, model) {
  checkNullArg(range, model)
  ch = isNumber(range) && range > 0 && range < max
  if(!ch) {
    msg = sprintf("For the `%s` model, `range` must be in the interval `(0,%g)`: ", model, max)
    stop2(msg)
  }
}

test_that("bad custom matrices are caught", {
  expect_error(mutationMatrix(),
               "`matrix` cannot be NULL with the `custom` model")
  expect_error(mutationMatrix(matrix=data.frame(a=1)),
               "Custom matrix must be a matrix, not a data.frame")
  expect_error(mutationMatrix(matrix=list(a=1)),
               "Custom matrix must be a matrix, not a list")
  expect_error(mutationMatrix(matrix=1),
               "Custom matrix must be a matrix, not a numeric")
  expect_error(mutationMatrix(matrix=matrix("a", ncol=1)),
               "Custom matrix must be numeric, not character")
  expect_error(mutationMatrix(matrix=matrix(0, nrow=2, ncol=1)),
               "Custom matrix must be square, not 2*1", fixed=T)
  expect_error(mutationMatrix(matrix=matrix(1, ncol=1), alleles=1:2),
               "Length of `alleles` must equal the dimension of `matrix`")
  expect_error(mutationMatrix(matrix=matrix(1, ncol=1)),
               "When custom matrix lacks names, the argument `alleles` cannot be NULL")
  m = matrix(1/2, 2, 2, dimnames = list(c("a", "b"), NULL))
  expect_error(mutationMatrix(matrix = m),
               "Mutation matrix has unequal row and column names")
})

test_that("bad inputs are caught", {
  expect_error(mutationMatrix(model="eq", matrix=matrix(1, ncol=1)),
               "`matrix` cannot be used with the `equal` model")
  expect_error(mutationMatrix(model="eq"),
               "`alleles` cannot be NULL with the `equal` model")
  expect_error(mutationMatrix(model="eq", alleles=1:2),
               "`rate` cannot be NULL with the `equal` model")
  expect_error(mutationMatrix(model="prop", alleles=1:2, rate=0),
               "`afreq` cannot be NULL with the `proportional` model")
  expect_error(mutationMatrix(model="prop", alleles=1:2, afreq=1),
               "Frequency vector does not match the number of alleles")
  expect_error(mutationMatrix(model="prop", alleles=1:2, afreq=c(0.5, 0.501)),
               "Allele frequencies do not sum to 1")
  expect_error(mutationMatrix(model="prop", alleles=1:2, afreq=c(.5,.5), rate=2),
               "`rate` must be a number in the interval")
  expect_error(mutationMatrix(model="prop", alleles=1:2, afreq=c(.3,.7), rate=1),
               "Model undefined: max `rate` for the given input is: 0.6")
  expect_error(mutationMatrix(model="step", alleles=1:2, rate=1, rate2=0.5, range=0.1),
               "The total mutation rate")
  expect_error(mutationMatrix(model="step", alleles=1:2, rate=0, rate2=0),
               "`range` cannot be NULL with the `stepwise` model")
  expect_error(mutationMatrix(model="dawid", alleles=1:2, afreq = c(.3,.7), rate=0, rate2=0, range=1),
               "For the `dawid` model, `range` must be in the interval")
})


test_that("equal model works", {
  expect_equivalent(
    mutationMatrix(model = "eq", alleles = 1:3, rate = 0),
    mutationMatrix(matrix = diag(3), alleles = 1:3))

  m = mutationMatrix(alleles = 1:3, model = "equal", rate = 0.1)
  expect_silent(validateMutationMatrix(m, alleles = 1:3))
  expect_equivalent(diag(m), rep(0.9, 3))
  expect_equivalent(rowSums(m), rep(1,3))

  expect_equivalent(m, mutationMatrix(matrix = m))
})

test_that("proportional model works", {
  afr = c(0.2, 0.3, 0.5)
  expect_equivalent(
    mutationMatrix(model = "prop", alleles = 1:3, rate = 0, afreq=afr),
    mutationMatrix(matrix = diag(3), alleles = 1:3))

  m = mutationMatrix(model = "prop", alleles = 1:3, rate = 0.1, afreq=afr)
  expect_silent(validateMutationMatrix(m, alleles = 1:3))
  expect_equivalent(rowSums(m), rep(1,3))

  expect_equal(c(m[2,1], m[1,2], m[1,3]), c(m[3,1], m[3,2], m[2,3]))

  expect_equivalent(m, mutationMatrix(matrix = m))
})

test_that("random model works", {
  m = mutationMatrix(alleles = 1:3, model = "random")
  expect_silent(validateMutationMatrix(m, alleles = 1:3))
  expect_equivalent(rowSums(m), rep(1,3))

  expect_equivalent(m, mutationMatrix(matrix = m))
})

test_that("trivial model works", {
  m = mutationMatrix(alleles = 1:3, model = "triv")
  expect_silent(validateMutationMatrix(m, alleles = 1:3))
  expect_equivalent(m, mutationMatrix(matrix = diag(3), alleles = 1:3))

  expect_equivalent(m, mutationMatrix(matrix = m))
})

test_that("trivial stepwise model is diagonal", {
  m = mutationMatrix(alleles = 1:3, model = "step",
                     rate=0, rate2=0, range=0.5)

  expect_silent(validateMutationMatrix(m, alleles = 1:3))
  expect_equivalent(m, mutationMatrix(matrix = diag(3), alleles = 1:3))
})

test_that("non-trivial stepwise mutation matrix is correct", {
  alleles = c(1, 1.5, 2,3)
  rate = 0.6
  rate2 = 0.3
  range = 0.5
  M = mutationMatrix("step", alleles=alleles, rate=rate, rate2=rate2, range = range)
  expect_equivalent(unclass(M),
                    matrix(c(.1,.1,.3,.2,.3,.7,.3,.3,.4,.1,.1,.4,.2,.1,.3,.1),
                           ncol=4, dimnames = list(alleles, alleles)))
})

#' Dawid's reversible stepwise model
#'
#' #' A reversible stepwise mutation model is created following the approach of
#' Dawid et al. (2002).
#'
#' **NB: This function is deprecated: Use `mutationMatrix(model = "dawid", ...)`
#' instead.**
#'
#' For the stepwise reversible model, the mutation rate \eqn{r_{i,j},\,  i\neq
#' j} is proportional to the overall mutation rate \eqn{\lambda} for given
#' values of the range, the allele frequency \eqn{p_i} and n, the number of
#' alleles. Hence, one can determine bounds UW and UB so that the model is well
#' defined if \eqn{\lambda \leq UW} and bounded, i.e., \eqn{r_{i,j} \leq p_j,\,
#' i\neq j}, if \eqn{\lambda \leq UB}, The bounds UW and UB are computed.
#'
#' @param alleles A vector of integer integers.
#' @param afreq A numeric vector of allele frequencies.
#' @param rate A numeric mutation rate.
#' @param range A positive number.
#' @param maxRateOnly A logical, by default FALSE. See Value.
#'
#' @return A reversible stepwise mutation model with overall mutation rate equal
#'   to `rate`.
#'
#'   If `maxRateOnly` is TRUE, the function returns a vector of two numbers
#'   named `UW` and `UB`. The first of these is the maximum overall mutation
#'   rate for a well-defined stepwise reversible mutation matrix with the given
#'   input. The latter (UB) is the maximum rate under the additional restraint
#'   that the model is bounded by `afreq`.
#'
#' @author Thore Egeland.
#'
#' @export
#'
#' @examples
#' stepwiseReversible(alleles = 1:3,
#'                    afreq = c(0.2, 0.3,  0.5),
#'                    rate = 0.001,
#'                    range = 0.1)
#'
#' stepwiseReversible(alleles = 1:3,
#'                    afreq = c(0.2, 0.3,  0.5),
#'                    range = 0.1,
#'                    maxRateOnly = TRUE)
#'
#' # Model not well defined:
#' \dontrun{
#' stepwiseReversible(alleles = 1:3,
#'                    afreq = c(0.2, 0.3,  0.5),
#'                    rate = 0.7,
#'                    range = 0.1)
#' }
stepwiseReversible = function(alleles, afreq, rate, range, maxRateOnly = FALSE) {
  if(!is.integer(alleles) && !(is.numeric(alleles) && all(alleles == as.integer(alleles))))
    stop2("Non-integer alleles detected")
  if(!is.numeric(range) || (range <= 0 || range >= 1))
    stop2("`range` must be in the interval (0,1): ", range)

  mxr = maxRate(alleles, afreq,  range)

  if(maxRateOnly)
    return(mxr)

  if(mxr[["UW"]] < rate)
    stop2("Model not well defined; max `rate` for the given input is: ", mxr[["UW"]])

  # remaining checking will be taken care of by `mutationModel` below
  n = length(afreq)
  a = (1 - range^n)/(1 - range)

  R = matrix(ncol = n, nrow = n, 0)
    for (i in 1:n){
      for(j in (1:n)[-i]) {

          R[i,j] = rate * (1 - range) * range^{abs(i-j)}/
                   (2*range*(n - a))*(1/afreq[i])
        }
      R[i,i] = 1 - sum(R[i,-i])
    }

  alleles = as.character(alleles)
  dimnames(R) = list(alleles, alleles)
  mutationMatrix(matrix = R, model = "custom", afreq = afreq, alleles = alleles)
}



#' Upper limits for overall mutation rate for the stepwise reversible model.
#'
#' @param alleles A character vector with allele labels.
#' @param afreq A numeric vector of allele frequencies.
#' @param range A positive number.
#'
#' @return A vector of two numbers named `UW` and `UB`. The first of these is
#'   the maximum overall mutation rate for a well-defined stepwise reversible
#'   mutation matrix with the given input. The latter (UB) is the upper limit of
#'   the overall mutation rate under the additional restraint that the model is
#'   bounded by `afreq`.
#'
#' @author Thore Egeland.
#'
maxRate = function(alleles, afreq,  range){
  n = length(afreq)

  R1 = matrix(ncol = n, nrow = n, 0)

  for (i in 1:n){
    for(j in setdiff(1:n, i)){
      a = (1 - range^n)/(1 - range)
      R1[i,j] = 1 * (1 - range) * range^{abs(i-j)}/
        (2*range*(n - a))*(1/afreq[i])
    }
  }

  # Essential that diag(R1) = 0
  linesums = apply(R1, 1, sum)
  boundDefined = 1/max(linesums)
  maks = apply(R1, 2, max)
  c(UW = boundDefined,
    UB = min(afreq/maks))
}

# Faster version of maxRate.
# Not currently used.
maxRateDawid = function(afreq, range){
  n = length(afreq)
  stepsize = outer(1:n, 1:n, function(i,j) abs(i-j))

  a = (1 - range^n) / (1 - range)
  R = (1 - range) / (2*range*(n - a)) * range^stepsize / afreq
  diag(R) = 0

  maxDefined = 1/max(rowSums(R))
  maxBounded = min(afreq / apply(R, 2, max))
  c(UW = maxDefined, UB = maxBounded)
}

#' Mutation model properties
#'
#' Functions that check various properties of a mutation model, including
#' stationarity, reversibility and lumpability.
#'
#' The function `isBounded()` checks that a mutation model is *bounded* by the
#' allele frequencies, i.e., that `mutmat[i,j] <= afreq[j]` whenever `i` is not
#' equal to `j`.
#'
#' Lumpability is a property of a mutation model that allows aggregating alleles
#' into groups, or *lumps*, without changing the overall mutation process. The
#' functions `isLumpable()` and `alwaysLumpable()` checks lumpability using the
#' row-sum criterion given by Kemeny & Snell (1976). Note that lumping may be
#' possible even if the model is not generally lumpable; see [lumpMutSpecial()]
#' for details.
#'
#' For each of these functions, if `mutmat` is a `mutationModel` object, i.e.,
#' with male and female components, the output is TRUE if and only if both
#' components satisfy the property in question.
#'
#' @param mutmat A [mutationMatrix()] or a [mutationModel()].
#' @param afreq A vector with allele frequencies, of the same length as the size
#'   of `mutmat`.
#' @param lump A character vector containing a nonempty set of allele labels.
#'
#' @return Each of these functions returns TRUE of FALSE.
#'
#' @references
#' Kemeny & Snell (1976). \emph{Finite Markov Chains}. Springer.
#'
#' @examples
#'
#' # "proportional" models are stationary and reversible
#' afr = c(0.2, 0.3, 0.5)
#' m_prop = mutationMatrix(model = "prop", alleles = 1:3, afreq = afr, rate = 0.1)
#' stopifnot(isStationary(m_prop, afr), isReversible(m_prop, afr))
#'
#' # "equal" model is stationary and reversible only when freqs are equal
#' m_eq = mutationMatrix(model = "eq", alleles = 1:3, rate = 0.1)
#' stopifnot(isStationary(m_eq, rep(1/3, 3)), isReversible(m_eq, rep(1/3, 3)))
#' stopifnot(!isStationary(m_eq, afr), !isReversible(m_eq, afr))
#'
#' # "equal" and "proportional" models allow allele lumping
#' stopifnot(isLumpable(m_eq, lump = 1:2))
#' stopifnot(isLumpable(m_prop, lump = 1:2))
#'
#' # In fact lumpable for any allele subset
#' stopifnot(alwaysLumpable(m_eq), alwaysLumpable(m_prop))
#'
#' @name model_properties
NULL

#' @rdname model_properties
#' @export
isStationary = function(mutmat, afreq = NULL) {
  if(isMutationModel(mutmat)) {
    isStatF = isStationary(mutmat$female, afreq = afreq)
    isStatM = sexEqual(mutmat) || isStationary(mutmat$male, afreq = afreq)
    return(isStatF && isStatM)
  }

  if(is.null(afreq))
    afreq = attr(mutmat, "afreq") %||% stop2("Argument `afreq` is missing")

  prod = as.numeric(afreq %*% mutmat)
  tol = sqrt(.Machine$double.eps)
  all(abs(as.numeric(afreq) - prod) < tol)
}

#' @rdname model_properties
#' @export
isReversible = function(mutmat, afreq = NULL) {
  if(isMutationModel(mutmat)) {
    isRevF = isReversible(mutmat$female, afreq = afreq)
    isRevM = sexEqual(mutmat) || isReversible(mutmat$male, afreq = afreq)
    return(isRevF && isRevM)
  }

  if(is.null(afreq))
    afreq = attr(mutmat, "afreq") %||% stop2("Argument `afreq` is missing")

  pm = afreq * mutmat
  pmT = t.default(pm)
  tol = sqrt(.Machine$double.eps)
  all(abs(as.numeric(pm) - as.numeric(pmT)) < tol)
}


#' @rdname model_properties
#' @export
isBounded = function(mutmat, afreq = NULL) {
  if(isMutationModel(mutmat)) {
    isboundF = isBounded(mutmat$female, afreq = afreq)
    isboundM = sexEqual(mutmat) || isBounded(mutmat$male, afreq = afreq)
    return(isboundF & isboundM)
  }

  if(is.null(afreq))
    afreq = attr(mutmat, "afreq") %||% stop2("Argument `afreq` is missing")

  n = length(afreq)
  M = as.matrix(mutmat)
  lines = rep(FALSE, n)
  for (i in 1:n)
    lines[i] = all(M[-i, i] <= afreq[i])
  all(lines)
}


#' @rdname model_properties
#' @export
isLumpable = function(mutmat, lump) {

  if(isMutationModel(mutmat)) {

    if(isTRUE(attr(mutmat, 'alwaysLumpable')))
      return(TRUE)

    test = isLumpable(mutmat$female, lump)
    if(!sexEqual(mutmat))
      test = test || isLumpable(mutmat$male, lump)

    return(test)
  }

  if(is.null(mutmat))
    return(TRUE)

  als = colnames(mutmat)
  lump = prepLump(lump, als)
  N = length(lump)

  if(N == 0)
    return(TRUE)

  tol = sqrt(.Machine$double.eps)

  if(N == 1) {
    lump = lump[[1]]

    # Exhaustive lump (trivial)
    if(length(lump) == length(als))
      return(TRUE)

    # Row sum criterion (reduced to equalities of entries)
    y = mutmat[lump, .mysetdiff(als, lump), drop = FALSE]
    checks = abs(as.numeric(y) - rep(y[1, ], each = length(lump))) < tol
    return(all(checks))
  }

  ### Multiple lumps (of size > 1) ###

  # Alleles not involved in lumps
  singles = .mysetdiff(als, unlist(lump, use.names = FALSE))

  # Loop through all lumps
  for(i in seq_len(N)) {
    a = lump[[i]]

    # Check against singles
    y = mutmat[a, singles, drop = FALSE]
    checks = abs(as.numeric(y) - rep(y[1, ], each = length(a))) < tol
    if(!all(checks))
      return(FALSE)

    # Check row sums against other lumps
    for(b in lump[-i]) {
      z = mutmat[a, b, drop = FALSE]
      rs = .rowSums(z, length(a), length(b))
      if(!.equalNums(rs, tol))
        return(FALSE)
    }
  }

  return(TRUE)
}

#' @rdname model_properties
#' @export
alwaysLumpable = function(mutmat) {

  if(isMutationModel(mutmat)) {
    alwaysF = alwaysLumpable(mutmat$female)
    return(alwaysF && (sexEqual(mutmat) || alwaysLumpable(mutmat$male)))
  }

  N = dim(mutmat)[1L] %||% 0L  # allows NULL input

  # 2*2 and smaller: trivially lumpable
  if(N <= 2)
    return(TRUE)

  offdiag = mutmat[col(mutmat) != row(mutmat)]
  dim(offdiag) = c(N - 1, N)

  # Kemeny & Snell criterion
  tol = sqrt(.Machine$double.eps)
  all(abs(as.numeric(offdiag) - rep(offdiag[1,], each = N-1)) < tol)
}


#' Find the stationary frequency distribution
#'
#' Finds the stationary distribution of allele frequencies, if it exists, w.r.t. a given mutation matrix.
#' @param mutmat A mutation matrix.
#'
#' @return A vector of length `ncol(mutmat)`, or NULL.
#'
#' @examples
#'
#' m1 = mutationMatrix("equal", alleles = 1:4, rate = 0.1)
#' findStationary(m1)
#'
#' m2 = mutationMatrix("random", alleles = 1:3, seed = 123)
#' a = findStationary(m2)
#'
#' a %*% m2 - a  # check
#'
#' @export
findStationary = function(mutmat) {
  validateMutationMatrix(mutmat)
  n = dim(mutmat)[1L]
  P = diag(n) - mutmat
  A = rbind(t.default(P), rep(1, n))
  b = c(rep(0, n), 1)

  # The following is effectively `qr.solve(A, b)`, but catches singular A gracefully
  QR = qr.default(A)

  if(QR$rank != n) {
    message("No stationary distribution")
    return()
  }

  qr.coef(QR, b)
}

test_that("equal model is station/revers only for equal freqs", {
  mutmat = mutationMatrix(alleles = 1:3, model = "equal", rate = 0.1)
  expect_false(isStationary(mutmat, c(0.2, 0.3, 0.5)))
  expect_false(isReversible(mutmat, c(0.2, 0.3, 0.5)))

  expect_true(isStationary(mutmat, c(1,1,1)/3))
  expect_true(isReversible(mutmat, c(1,1,1)/3))
})

test_that("proportial model is stationary and reversible", {
  afreq = c(0.2, 0.3, 0.5)
  mutmat = mutationMatrix(alleles = 1:3, model = "prop", rate = 0.1, afreq = afreq)
  expect_true(isStationary(mutmat, afreq))
  expect_true(isReversible(mutmat, afreq))
})

test_that("random model 3*3 has correct rate", {
  afreq = c(a=0.2, b=0.3, c=0.5)
  m1 = mutationMatrix(afreq = afreq, model = "random", rate = 0.001, seed = 1)
  expect_equal(mutRate(m1), 0.001)
  m2 = mutationMatrix(afreq = afreq, model = "random", rate = 0.999, seed = 1)
  expect_equal(mutRate(m2), 0.999)
})

test_that("equal model is always lumpable", {
  mutmat = mutationMatrix(alleles = 1:4, model = "equal", rate = 0.1)
  expect_true(isLumpable(mutmat, lump = 1:2))
  expect_true(isLumpable(mutmat, lump = c(2,4)))
  expect_true(isLumpable(mutmat, lump = c(1:2,4)))
})

test_that("proportional model is always lumpable", {
  afreq = c(0.2, 0.3, 0.5)
  mutmat = mutationMatrix(alleles = 1:3, model = "prop", rate = 0.1, afreq = afreq)
  expect_true(isLumpable(mutmat, lump = 1:2))
  expect_true(isLumpable(mutmat, lump = c(1,3)))
})

test_that("isStationary and isReversible works with full models", {
  afreq = c(0.5, 0.5)
  m1 = mutationMatrix("eq", alleles = 1:2, rate = 0.1, afreq = afreq)
  m2 = mutationMatrix("random", alleles = 1:2, afreq = afreq)
  expect_true(isStationary(mutationModel(list(female = m1, male = m1))))
  expect_false(isStationary(mutationModel(list(female = m1, male = m2))))
  expect_false(isStationary(mutationModel(list(female = m2, male = m2))))

  expect_true(isReversible(mutationModel(list(female = m1, male = m1))))
  expect_false(isReversible(mutationModel(list(female = m1, male = m2))))
  expect_false(isReversible(mutationModel(list(female = m2, male = m2))))

  stb = stabilize(mutationModel(list(female = m1, male = m2)))
  expect_true(isStationary(stb))
})

#' Special lumping of mutation models
#'
#' This function implements methods for special, or pedigree-aware, allele
#' lumping. This is typically attempted if the model is not generally lumpable
#' as determined by [alwaysLumpable()]. Note that the resulting lumped model is
#' tailor-made for a specific likelihood calculation, and may violate the
#' properties of a well-defined mutation model.
#'
#' The lumping procedure depends on the location of untyped individuals in the
#' pedigree, summarised by the so-called U-signature:
#'
#' * F-depth: The length of the longest chain of untyped, starting with a founder
#' * F-width: The maximum number of children of an untyped founder
#' * N-depth: The length of the longest chain of untyped, starting with a nonfounder
#' * N-width: The maximum number of children of an untyped nonfounder
#'
#' @param mut A square mutation matrix; typically a [mutationMatrix()] or
#'   [mutationModel()].
#' @param lump A vector containing the alleles to be lumped together.
#' @param uSign The U-signature of the pedigree for which lumping is attempted.
#'   See Details.
#' @param afreq A vector with allele frequencies, of the same length as the size
#'   of `mut`. Extracted from the model if not given.
#' @param verbose A logical.
#'
#' @returns A reduced mutation model, if lumping was possible, otherwise the
#'   original model is returned unchanged.
#'
#' @seealso [lumpedModel()].
#'
#' @examples
#'
#' af = rep(0.05, 20)
#' names(af) = 1:20
#' m = mutationMatrix("random", afreq = af, rate = 0.1, seed = 1)
#'
#' # Degree 1 lumping
#' mL = lumpMutSpecial(m, lump = 3:20, uSign = c(1,1,0,0))
#' mL
#'
#' # Check
#' afL = attr(mL, "afreq")
#' stopifnot(sum(af * m[, 1]) == sum(afL * mL[, 1]))
#'
#' # Degree 2
#' mL2 = lumpMutSpecial(m, lump = 3:20, uSign = c(1,2,0,0))
#' mL2
#' afL2 = attr(mL2, "afreq")
#'
#' stopifnot(all.equal(af %*% m[, 1]^2, afL2 %*% mL2[, 1]^2),
#'           all.equal(af %*% m[, 2]^2, afL2 %*% mL2[, 2]^2),
#'           all.equal(af %*% ( m[, 1]*m[, 2]),
#'                     afL2 %*% (mL2[, 1]*mL2[, 2])))
#'
#' @importFrom lpSolve lp
#' @export
lumpMutSpecial = function(mut, lump, uSign, afreq = NULL, verbose = TRUE) {

  isMod = isMutationModel(mut)
  sexeq = isMod && sexEqual(mut)

  # Extract frequencies (a bit ad hoc)
  afreq = afreq %||% attr(mut, "afreq") %||% attr(mut$female, "afreq") %||%
    stop2("`afreq` must be provided")

  als = if(isMod) colnames(mut$male) else colnames(mut)
  lump = prepLump(lump, alleles = als)[[1]]

  if(verbose) {
    nms = names(uSign)
    us = if(is.null(nms)) toString(uSign) else paste(nms, uSign, sep = "=", collapse = ", ")
    cat("U-signature:", us, "\n")
  }

  # Founder chains > 1, and nonfounder-chains not implemented yet
  if(uSign[1] > 1 || uSign[3] + uSign[4] > 0) {
    if(verbose) cat("Lumping not implemented for this signature - returning unchanged\n")
    return(mut)
  }

  # By now: uSign = c(1,d,0,0). All untyped are founders, with max d children
  d = uSign[2]

  if(d == 1) {
    if(verbose) cat("Lumping method: Direct\n")

    .directLump = function(mutmat, afreq, lump) {
      keep = .mysetdiff(names(afreq), lump)
      newAls = c(keep, "lump")
      newAfreq = c(afreq[keep], lump = sum(afreq[lump]))

      N = length(newAls)
      wei = afreq[lump]/sum(afreq[lump])

      newM = matrix(0, ncol = N, nrow = N, dimnames = list(newAls, newAls))
      newM[keep, keep] = mutmat[keep, keep]
      newM[N, -N] = as.numeric(wei %*% mutmat[lump, keep, drop = FALSE])
      newM[, N] = 1 - .rowSums(newM, N, N)
      newMutationMatrix(newM, afreq = newAfreq, lumpedAlleles = lump,
                   model = paste0(attr(mutmat, 'model'), ", lumped"))
    }

    if(!isMod) res = .directLump(mut, afreq, lump)
    else res = mapFullModel(mut, .directLump, afreq = afreq, lump = lump)
    return(res)
  }
  else {
    if(verbose) cat("Lumping method: LP\n")

    keep = .mysetdiff(als, lump)
    nk = length(keep)
    p = afreq[lump]

    # Main computation: Select subset of rows to keep
    A = (if(isMod) mut$female else mut)[lump, keep, drop = FALSE]
    B = if(isMod) mut$male[lump, keep, drop = FALSE] else NULL
    red = reduceLP(p, A = A, B = B, d = d, verbose = FALSE)

    # Was the reduction successful?
    if(red$status != 0) {
      if(verbose) cat("Lumping unsuccessful: No solution found\n")
      return(mut)
    }

    np = length(red$p)
    if(np == length(lump)) {
      if(verbose) cat("Lumping unsuccessful: No reduction\n")
      return(mut)
    }

    newAls = c(keep, paste0(".lmp", 1:np))
    newAfreq = c(afreq[keep], red$p)
    names(newAfreq) = newAls
    N = length(newAls)

    # Create reduced matrix; R is "lumped rectangle"
    .createLPmat = function(mutmat, R) {
      newM = matrix(0, ncol = N, nrow = N, dimnames = list(newAls, newAls))
      newM[1:nk, 1:nk] = mutmat[keep, keep]
      newM[nk + 1:np, 1:nk] = R
      newM[ , N] = 1 - rowSums(newM)
      newMutationMatrix(newM, afreq = newAfreq, lumpedAlleles = lump,
                        model = paste0(attr(mutmat, 'model'), ", lumped"))
    }

    # If single matrix, just returned the reduced matrix
    if(!isMod)
      return(.createLPmat(mut, red$A))

    # If full model, return new model
    matF = .createLPmat(mut$female, red$A)
    matM = .createLPmat(mut$male, red$B)
    lumpable = alwaysLumpable(matF) && (sexeq || alwaysLumpable(matM))
    res = structure(list(female = matF, male = matM),
              afreq = newAfreq, sexEqual = sexeq,
              alwaysLumpable = lumpable, class = "mutationModel")
    return(res)
  }
}


# Helper functions for special lumping ------------------------------------

reduceLP = function(p, A, B = NULL, d, verbose = TRUE) {
  n = length(p); scale = sum(p); p1 = p/scale

  # Monomial exponents: All tuples summing to at most d
  comb = as.matrix(expand.grid(rep(list(0:d), ncol(A))))
  comb = comb[rowSums(comb) %in% 1:d, , drop=FALSE]

  # Find monomials (of columns of A) of degree up to d
  monomsFix0 = function(Y) {
    if(all(Y > 0))
      return(exp(comb %*% t(log(Y))))

    # Temporarily replace 0 with 1 in log(Y) to avoid -Inf
    logY = log(replace(Y, Y == 0, 1))
    zeros = (comb > 0) %*% t.default(Y == 0) > 0
    monoms = exp(comb %*% t.default(logY))
    monoms[zeros] = 0
    monoms
  }

  monomsA = monomsFix0(A)
  M = rbind(1, monomsA)
  rhs = c(1, monomsA %*% p1)

  # If B is not NULL, add monomials of B
  if(!is.null(B)) {
    monomsB = monomsFix0(B)
    M = rbind(M, monomsB)
    rhs = c(rhs, monomsB %*% p1)
  }

  # Row-scale each constraint
  scales = apply(abs(M), 1, max)
  scales[scales == 0] = 1
  M1 = M / scales
  rhs1 = rhs / scales

  sol = lpSolve::lp("min", rep(0, n), M1, "=", rhs1, scale = 0)
  keep = which(sol$solution > 1e-10)

  res = list(status = sol$status,
             p = scale * sol$solution[keep],
             A = A[keep, , drop=FALSE])

  if(!is.null(B))
    res$B = B[keep, , drop=FALSE]

  res
}

# Stroud lumping  ---------------------------------------------------------

# Not working well: Problems with rank deficiency and numerical instability)

# A centred simplex in R^n
simplex = function(n) {
  N = n + 1
  # Vertices in R^N: e_i - (1/N)*1, for i = 1,..., N
  V = diag(N) - 1/N
  # Orthonormal basis for the subspace where sum = 0
   Q = qr.Q(qr(matrix(1, N, 1)), complete = TRUE)[, 2:N]  # Q is N x n.
  # Vertices in R^n: n x N
  coords = t(Q) %*% V
  # Scale so that, with equal weights 1/(n+1), the covariance is I_n:
  coords * sqrt(N)
}

# Not used: Non-centered simplex. See Stroud's book page 161.
simplex2 = function(n) {
  r = (n + 2 - sqrt(n+2))/((n+1)*(n+2))
  s = (n + 2 + sqrt(n+2))/((n+1)*(n+2))
  res = matrix(r, n, n+1)
  res[cbind(1:n, 2:(n+1))] = s
  res
}

stroud = function(U, P) {
  sumP = sum(P)
  P = P / sumP
  m = as.numeric(U %*% P)
  n = length(m)
  C = U %*% diag(P) %*% t(U) - m %*% t(m)
  # C = C + 1e-10*diag(n)
  # C = as.matrix(nearPD(C)$mat)
  L = t(chol(C))
  nodes = L %*% simplex(n) + m
  list(nodes = nodes, weights = rep(sumP/(n+1), n+1))
}

# Code used in lumpMutSpecial:
#
# if(FALSE && d == 2) {
#   if(length(als) <= 2*nKeep + 1) {
#     if(verbose) cat("Nothing to gain by lumping\n")
#     return(mutmat)
#   }
#
#   # Main computation: Find nodes
#   U = mutmat[lump, keep, drop = FALSE]
#   reduced = stroud(t(U), afreq[lump])
#
#   # Replace lumped alleles with nKeep + 1 nodes
#   nNodes = nKeep + 1
#   nodeNames = paste0(".lmp", 1:nNodes)
#   newAls = c(keep, nodeNames)
#
#   # Compute the new mutation matrix
#   N = nKeep + nNodes
#   newM = matrix(0, ncol = N, nrow = N, dimnames = list(newAls, newAls))
#   newM[1:nKeep, 1:nKeep] = mutmat[keep, keep]
#   newM[nKeep + (1:nNodes), 1:nKeep] = t(reduced$nodes)
#
#   # New frequency vector
#   lumpFreq = (1 - sum(afreq[keep]))/nNodes
#   newAfreq = as.numeric(c(afreq[keep], rep(lumpFreq, nNodes)))
#   names(newAfreq) = newAls
# }

#' Transformation (stabilisation) to stationarity
#'
#' For a given mutation model `(M,p)`, transform `M` into another mutation
#' matrix `S` such that `S` is stationary with respect to `p`. Several methods
#' for doing this are described by Simonsson and Mostad (2016); only the "PM"
#' method is included here.
#'
#' These transformations may also be applied by setting `transform = "PM"` in
#' [mutationMatrix()] or [mutationModel()].
#'
#' For details about the transformation, see Simonsson and Mostad (2016).
#'
#' This function is a slightly optimised version of the `stabilize()` method in
#' the Familias R package.
#'
#' @param mutmat A square mutation matrix; typically a [mutationMatrix()] or
#'   [mutationModel()].
#' @param method A character string indicating the method to use. Currently only
#'   "PM" is implemented.
#' @param afreq A vector of allele frequencies. Extracted from `mutmat` if not
#'   provided.
#'
#' @return An object of the same class the input `mutmat`; either a matrix, a
#'   `mutationMatrix` or a `mutationModel`.
#'
#' @references Simonsson & Mostad (2016). *Stationary mutation models*. Forensic
#'   Sci. Int. Genet. 23:217–225. \doi{10.1016/j.fsigen.2016.04.005}
#'
#' @examples
#'
#' afreq = c(`1` = .2, `2` = .3, `3` = .5)
#' m = mutationMatrix("step", afreq = afreq, rate=0.1, rate2=0.01, range=0.1)
#' m
#' makeStationary(m, afreq = c(.3,.3,.4))
#'
#'
#' ### Example with full model (i.e., male and female)
#'
#' M = mutationModel("equal", afreq = afreq, rate = list(male=0.1, female=0.2))
#' M
#' makeStationary(M)
#'
#' @export
makeStationary = function(mutmat, afreq = NULL, method = "PM") {

  if(isMutationModel(mutmat))
    return(mapFullModel(mutmat, makeStationary, afreq = afreq, method = method))

  if(!is.matrix(mutmat))
    stop2("Argument `mutmat` must be either a `matrix`, a `mutationMatrix()` or a `mutationModel()`: ", class(mutmat)[1])

  method = match.arg(method)
  afreq = afreq %||% attr(mutmat, "afreq") %||% stop2("`afreq` must be provided")

  S = .stabilizePM(mutmat, afreq)
  newrate = mutRate(mutmat, afreq)

  newMutationMatrix(S, "custom", afreq = afreq, rate = newrate)
}


# Based on Familias::stabilize
.stabilizePM = function(M, p) {

  n = dim(M)[1]
  dM = diag(M)
  I = diag(n)

  R = 1 - sum(dM * p)

  X = t.default(M) - I
  X[n, ] = 1

  v = solve.default(X, c(rep(0, n-1), 1))
  d = R * v / ((1 - sum(dM * v)) * p)

  if(any(d * (1 - dM) > 1))
    stop2("PM stabilization doesn't exist.")

  (M - I) * d + I
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stabilize.R
\name{stabilize}
\alias{stabilize}
\title{Stabilization of mutation matrix}
\usage{
stabilize(mutmat, afreq = NULL, method = "PM", details = FALSE)
}
\arguments{
\item{mutmat}{A mutation matrix.}

\item{afreq}{A vector of allele frequencies.}

\item{method}{Either "DP", "RM" or "PM". Currently only "PM" is implemented.}

\item{details}{A logical. If TRUE, the complete Familias output is included.}
}
\value{
An object of the same class the input \code{mutmat}; either a matrix, a
\code{mutationMatrix} or a \code{mutationModel}.
}
\description{
NB: REPLACED BY \link{makeStationary}. Produces a mutation matrix close to the
input \code{mutmat}, for which the given frequency vector is the stationary
distribution. Several methods for doing this are described by Simonsson and
Mostad (2016); only the "PM" method is included here.
}
\details{
This function is based on, and reuses code from, the \code{stabilize()} method of
the Familias R package.
}
\examples{

afreq = c(`1` = .2, `2` = .3, `3` = .5)
m = mutationMatrix("stepwise", afreq = afreq,
                   rate = 0.1, rate2 = 0.01, range = 0.1)
m
stabilize(m)


### Example with full model (i.e., male and female)

M = mutationModel("stepwise", alleles = 1:3, afreq = afreq,
                   rate = list(male = 0.1, female = 0.2),
                   rate2 = 0.01, range = 0.1)
M
stabilize(M)

}
\references{
Simonsson, Mostad: Stationary Mutation models. (FSI: Genetics,
2016).
}
\author{
Petter Mostad, Thore Egeland, Ivar Simonsson, Magnus D. Vigeland
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeStationary.R
\name{makeStationary}
\alias{makeStationary}
\title{Transformation (stabilisation) to stationarity}
\usage{
makeStationary(mutmat, afreq = NULL, method = "PM")
}
\arguments{
\item{mutmat}{A square mutation matrix; typically a \code{\link[=mutationMatrix]{mutationMatrix()}} or
\code{\link[=mutationModel]{mutationModel()}}.}

\item{afreq}{A vector of allele frequencies. Extracted from \code{mutmat} if not
provided.}

\item{method}{A character string indicating the method to use. Currently only
"PM" is implemented.}
}
\value{
An object of the same class the input \code{mutmat}; either a matrix, a
\code{mutationMatrix} or a \code{mutationModel}.
}
\description{
For a given mutation model \verb{(M,p)}, transform \code{M} into another mutation
matrix \code{S} such that \code{S} is stationary with respect to \code{p}. Several methods
for doing this are described by Simonsson and Mostad (2016); only the "PM"
method is included here.
}
\details{
These transformations may also be applied by setting \code{transform = "PM"} in
\code{\link[=mutationMatrix]{mutationMatrix()}} or \code{\link[=mutationModel]{mutationModel()}}.

For details about the transformation, see Simonsson and Mostad (2016).

This function is a slightly optimised version of the \code{stabilize()} method in
the Familias R package.
}
\examples{

afreq = c(`1` = .2, `2` = .3, `3` = .5)
m = mutationMatrix("step", afreq = afreq, rate=0.1, rate2=0.01, range=0.1)
m
makeStationary(m, afreq = c(.3,.3,.4))


### Example with full model (i.e., male and female)

M = mutationModel("equal", afreq = afreq, rate = list(male=0.1, female=0.2))
M
makeStationary(M)

}
\references{
Simonsson & Mostad (2016). \emph{Stationary mutation models}. Forensic
Sci. Int. Genet. 23:217–225. \doi{10.1016/j.fsigen.2016.04.005}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjustRate.R
\name{adjustRate}
\alias{adjustRate}
\title{Adjust the overall mutation rate of a model}
\usage{
adjustRate(mutmat, newrate, afreq = NULL, rate = NULL)
}
\arguments{
\item{mutmat}{A mutation matrix with nonzero mutation overall rate.}

\item{newrate}{The new overall mutation rate.}

\item{afreq}{The allele frequencies. Extracted from the mutation matrix if
not provided.}

\item{rate}{The current overall mutation rate. Calculated from the input if
not provided.}
}
\value{
A new mutation matrix with the adjusted rate.
}
\description{
Adjusts the overall mutation rate of a model by scaling the off-diagonal
matrix entries.
}
\details{
The adjusted matrix is calculated as \code{a * M + (1-a) * I}, where \code{M} is the
original matrix, \code{a = newrate/rate}, and \code{I} is the identity matrix.

The maximum allowed value of \code{newrate} (to avoid negative values in the
adjusted matrix) is \verb{rate/(1 - m))}, where \code{m} is the smallest diagonal
element in the original matrix.
}
\examples{
m = mutationMatrix("equal", afreq = c(a=0.2, b=0.3, c=0.5), rate = 0.2)
m
adjustRate(m, 0.4)

}
\seealso{
\code{\link[=mutRate]{mutRate()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_properties.R
\name{findStationary}
\alias{findStationary}
\title{Find the stationary frequency distribution}
\usage{
findStationary(mutmat)
}
\arguments{
\item{mutmat}{A mutation matrix.}
}
\value{
A vector of length \code{ncol(mutmat)}, or NULL.
}
\description{
Finds the stationary distribution of allele frequencies, if it exists, w.r.t. a given mutation matrix.
}
\examples{

m1 = mutationMatrix("equal", alleles = 1:4, rate = 0.1)
findStationary(m1)

m2 = mutationMatrix("random", alleles = 1:3, seed = 123)
a = findStationary(m2)

a \%*\% m2 - a  # check

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mutRate.R
\name{mutRate}
\alias{mutRate}
\title{Overall mutation rate}
\usage{
mutRate(mutmat, afreq = NULL)
}
\arguments{
\item{mutmat}{A \code{\link[=mutationMatrix]{mutationMatrix()}} or \code{\link[=mutationModel]{mutationModel()}}.}

\item{afreq}{A vector of allele frequencies.}
}
\value{
A single number, or (if \code{mutmat} is a \code{\link[=mutationModel]{mutationModel()}} and the female
and male rates differ) a list of two numbers, named "female" and "male".
}
\description{
Calculate the overall mutation rate at a locus, given a mutation model an a
set of allele frequencies.
}
\details{
The mutation rate is found by the formula \code{1 - sum(diag(mutmat) * afreq)}.

If \code{mutmat} is a full \code{\link[=mutationModel]{mutationModel()}}, the rate is calculated separately for
the male and female matrices.
}
\examples{
m = mutationMatrix("stepwise", alleles = 1:4, afreq = c(.1,.2,.3,.4),
                   rate = 0.01, rate2 = 1e-6, range = 0.1)
r = mutRate(m)

stopifnot(all.equal(r, 0.01))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mutationModel.R
\name{mutationModel}
\alias{mutationModel}
\alias{validateMutationModel}
\alias{sexEqual}
\title{Mutation models}
\usage{
mutationModel(
  model,
  alleles = NULL,
  afreq = NULL,
  matrix = NULL,
  rate = NULL,
  rate2 = NULL,
  range = NULL,
  seed = NULL,
  transform = NULL,
  validate = TRUE
)

validateMutationModel(mutmod, alleles = NULL)

sexEqual(mutmod)
}
\arguments{
\item{model}{Either:
\itemize{
\item a \code{mutationModel} object (returned unchanged after validation)
\item a single \code{mutationMatrix} object (will be applied to both genders)
\item a list of two \code{mutationMatrix} objects, named "female" and "male"
\item a single model name (see \code{\link[=mutationMatrix]{mutationMatrix()}} for valid options)
\item a list of two model names, named "female" and "male"
}}

\item{alleles}{A character vector with allele labels; passed on to
\code{\link[=mutationMatrix]{mutationMatrix()}}.}

\item{afreq}{A numeric vector of allele frequencies; passed on to
\code{\link[=mutationMatrix]{mutationMatrix()}}.}

\item{matrix}{A matrix, or a list of two matrices (named "female" and "male")}

\item{rate}{A numeric mutation rate, or a list of two (named "female" and
"male")}

\item{rate2}{A numeric mutation rate, or a list of two (named "female" and
"male"). Required in the "stepwise" model; see \code{\link[=mutationMatrix]{mutationMatrix()}} for
details.}

\item{range}{A positive number, or a list of two (named "female" and "male").
Required in the "stepwise" model; see \code{\link[=mutationMatrix]{mutationMatrix()}} for details.}

\item{seed}{An integer, or a list of two (named "female" and "male").}

\item{transform}{Either NULL (default) or one of the strings "MH", "BA",
"PR", "PM". See \code{\link[=mutationMatrix]{mutationMatrix()}}.}

\item{validate}{A logical, by default TRUE.}

\item{mutmod}{A \code{mutationModel} object.}
}
\value{
An object of class \code{mutationModel}. This is a list of two
\code{mutationMatrix} objects, named "female" and "male", and the following
attributes:
\itemize{
\item \code{sexEqual}: TRUE if both genders have identical models.
\item \code{alwaysLumpable}: TRUE if both genders have models that are lumpable for
any allele subset.
}
}
\description{
Constructor for the class \code{mutationModel}. An object of this class is
essentially a list of two mutation matrices, named "female" and "male".
}
\examples{
# "Equal" model, same parameters for both genders
M1 = mutationModel("eq", alleles = 1:2, rate = 0.1)
M1

# Different mutation rates
M2 = mutationModel("eq", alleles = 1:2, rate = list(male = 0.1, female = 0.01))
M2

stopifnot(identical(M1$male, M1$female), identical(M2$male, M1$male))

# A custom mutation matrix:
mat = matrix(c(0,0,1,1), ncol = 2, dimnames = list(1:2, 1:2))
M3 = mutationModel(model = "custom", matrix = mat)

# Under the hood arguments are passed to `mutationMatrix()`.
# Alternatively, this can be done explicitly in the `model` argument
M4 = mutationModel(model = mutationMatrix("custom", matrix = mat))

stopifnot(identical(M3, M4))

# The latter strategy is needed e.g. in `pedtools::marker()`, which gives the
# user access to `model`, but not `matrix`.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{isMutationModel}
\alias{isMutationModel}
\alias{isMutationMatrix}
\title{Test for mutation matrix/model}
\usage{
isMutationModel(x)

isMutationMatrix(x)
}
\arguments{
\item{x}{Any object.}
}
\value{
TRUE or FALSE
}
\description{
Test for mutation matrix/model
}
\examples{

mat = mutationMatrix("equal", alleles = 1:2, rate = 0.1)
isMutationMatrix(mat)

isMutationModel(mat) # FALSE (not a complete model)

mod = mutationModel(mat)
isMutationModel(mod)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pedmut-package.R
\docType{package}
\name{pedmut-package}
\alias{pedmut}
\alias{pedmut-package}
\title{pedmut: Mutation Models for Pedigree Likelihood Computations}
\description{
A collection of functions for modelling mutations in pedigrees with marker data, as used e.g. in likelihood computations with microsatellite data. Implemented models include equal, proportional and stepwise models, as well as random models for experimental work, and custom models allowing the user to apply any valid mutation matrix. Allele lumping is done following the lumpability criteria of Kemeny and Snell (1976), ISBN:0387901922.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/magnusdv/pedmut}
}

}
\author{
\strong{Maintainer}: Magnus Dehli Vigeland \email{m.d.vigeland@medisin.uio.no} (\href{https://orcid.org/0000-0002-9134-4962}{ORCID})

Authors:
\itemize{
  \item Magnus Dehli Vigeland \email{m.d.vigeland@medisin.uio.no} (\href{https://orcid.org/0000-0002-9134-4962}{ORCID})
}

Other contributors:
\itemize{
  \item Thore Egeland \email{thore.egeland@nmbu.no} (\href{https://orcid.org/0000-0002-3465-8885}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeReversible.R
\name{makeReversible}
\alias{makeReversible}
\title{Transformations to reversibility}
\usage{
makeReversible(
  mutmat,
  method = c("BA", "MH", "PR"),
  adjust = TRUE,
  afreq = NULL
)
}
\arguments{
\item{mutmat}{A \code{\link[=mutationMatrix]{mutationMatrix()}} or \code{\link[=mutationModel]{mutationModel()}}.}

\item{method}{A character indicating which transformation to use. Either "BA"
(Barker), "MH" (Metropolis-Hastings) or "PR" (preserved rate).}

\item{adjust}{Logical. If TRUE (default), the overall mutation rate is
adjusted to preserve the original rate; see \code{\link[=adjustRate]{adjustRate()}}. Not relevant
for method "PR", which by construction always preserves the overall rate.}

\item{afreq}{A vector of allele frequencies. Extracted from \code{mutmat} if not
provided.}
}
\value{
A reversible mutation matrix with the same allele frequencies.
}
\description{
This function implements three methods for transforming a mutation model
\verb{(M,p)} into a reversible one, \verb{(R,p)}. All methods are based on Metropolis-
Hastings proposal functions.
}
\details{
These transformations may also be applied through the \code{transform} argument of
\code{\link[=mutationMatrix]{mutationMatrix()}} and \code{\link[=mutationModel]{mutationModel()}}.
}
\examples{
m = mutationMatrix("equal", afreq = c(a=0.2, b=0.3, c=0.5), rate = 0.2)
makeReversible(m, "BA")
makeReversible(m, "MH")
makeReversible(m, "PR")

makeReversible(m, "BA", adjust = FALSE)  # rate differs!

# Apply to full model with different female/male rates
mod = mutationModel("equal", afreq = c(a=0.2, b=0.3, c=0.5),
                    rate = list(female = 0.1, male = 0.2))
modR = makeReversible(mod)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwiseReversible.R
\name{stepwiseReversible}
\alias{stepwiseReversible}
\title{Dawid's reversible stepwise model}
\usage{
stepwiseReversible(alleles, afreq, rate, range, maxRateOnly = FALSE)
}
\arguments{
\item{alleles}{A vector of integer integers.}

\item{afreq}{A numeric vector of allele frequencies.}

\item{rate}{A numeric mutation rate.}

\item{range}{A positive number.}

\item{maxRateOnly}{A logical, by default FALSE. See Value.}
}
\value{
A reversible stepwise mutation model with overall mutation rate equal
to \code{rate}.

If \code{maxRateOnly} is TRUE, the function returns a vector of two numbers
named \code{UW} and \code{UB}. The first of these is the maximum overall mutation
rate for a well-defined stepwise reversible mutation matrix with the given
input. The latter (UB) is the maximum rate under the additional restraint
that the model is bounded by \code{afreq}.
}
\description{
#' A reversible stepwise mutation model is created following the approach of
Dawid et al. (2002).
}
\details{
\strong{NB: This function is deprecated: Use \code{mutationMatrix(model = "dawid", ...)}
instead.}

For the stepwise reversible model, the mutation rate \eqn{r_{i,j},\,  i\neq
j} is proportional to the overall mutation rate \eqn{\lambda} for given
values of the range, the allele frequency \eqn{p_i} and n, the number of
alleles. Hence, one can determine bounds UW and UB so that the model is well
defined if \eqn{\lambda \leq UW} and bounded, i.e., \eqn{r_{i,j} \leq p_j,\,
i\neq j}, if \eqn{\lambda \leq UB}, The bounds UW and UB are computed.
}
\examples{
stepwiseReversible(alleles = 1:3,
                   afreq = c(0.2, 0.3,  0.5),
                   rate = 0.001,
                   range = 0.1)

stepwiseReversible(alleles = 1:3,
                   afreq = c(0.2, 0.3,  0.5),
                   range = 0.1,
                   maxRateOnly = TRUE)

# Model not well defined:
\dontrun{
stepwiseReversible(alleles = 1:3,
                   afreq = c(0.2, 0.3,  0.5),
                   rate = 0.7,
                   range = 0.1)
}
}
\author{
Thore Egeland.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lumpedModel.R
\name{lumpedMatrix}
\alias{lumpedMatrix}
\alias{lumpedModel}
\title{Combine alleles in a mutation matrix}
\usage{
lumpedMatrix(mutmat, lump, afreq = NULL, check = TRUE, labelSep = NULL)

lumpedModel(mutmod, lump, afreq = attr(mutmod, "afreq"), check = TRUE)
}
\arguments{
\item{mutmat}{A \code{mutationMatrix} object, typically made with
\code{\link[=mutationMatrix]{mutationMatrix()}}.}

\item{lump}{A vector containing the alleles to be lumped together, or a list
of several such vectors.}

\item{afreq}{A vector with allele frequencies, of the same length as the size
of \code{mutmat}. Extracted from the model if not given.}

\item{check}{A logical indicating if lumpability (i.e., the row-sum criterium
of Kemeny & Snell) should be checked before lumping. Default: TRUE.}

\item{labelSep}{A character used to name lumps by pasting allele labels. (For
debugging.)}

\item{mutmod}{A \code{mutationModel} object, typically made with
\code{\link[=mutationModel]{mutationModel()}}.}
}
\value{
A reduced mutation model. If the original matrix has dimensions
\eqn{n\times n}{n*n}, the result will be \eqn{k\times k}{k*k}, where \eqn{k
  = n - length(lump) + 1}.
}
\description{
Reduce a mutation matrix by combining a set of alleles into one \emph{lump}, if
this can be done without distorting the mutation process of the remaining
alleles. Such allele lumping can give dramatic efficiency improvements in
likelihood computations with multi-allelic markers, in cases where only some
of the alleles are observed in the pedigree.
}
\details{
The lumping implemented in this function is based on the Markov chain lumping
theory by Kemeny & Snell (1976). For other, specialised lumping, see
\code{\link[=lumpMutSpecial]{lumpMutSpecial()}}.
}
\examples{

af = c(.1, .2, .3, .4)
names(af) = 1:4

### Example 1: Lumping a mutation matrix
mat = mutationMatrix("eq", afreq = af, rate = 0.1)
mat

# Lump
lumpedMatrix(mat, lump = 3:4)
lumpedMatrix(mat, lump = 2:4)

# Example 2: Full model, proportional
mutrate = list(male = 0.1, female = 0.2)
mod = mutationModel("prop", afreq = af, rate = mutrate)
mod

# Lump
lumpedModel(mod, lump = 2:4)

}
\references{
Kemeny & Snell (1976). \emph{Finite Markov Chains}. Springer.
}
\seealso{
\code{\link[=lumpMutSpecial]{lumpMutSpecial()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mutationMatrix.R
\name{mutationMatrix}
\alias{mutationMatrix}
\alias{validateMutationMatrix}
\title{Mutation matrix}
\usage{
mutationMatrix(
  model = c("custom", "dawid", "equal", "proportional", "random", "onestep", "stepwise",
    "trivial"),
  matrix = NULL,
  alleles = NULL,
  afreq = NULL,
  rate = NULL,
  seed = NULL,
  rate2 = NULL,
  range = NULL,
  transform = NULL,
  validate = TRUE
)

validateMutationMatrix(mutmat, alleles = NULL)
}
\arguments{
\item{model}{A string: either "custom", "dawid", "equal", "proportional",
"random", "onestep", "stepwise" or "trivial".}

\item{matrix}{When \code{model} is "custom", this must be a square matrix with
nonnegative real entries and row sums equal to 1.}

\item{alleles}{A character vector (or coercible to character) with allele
labels. Required in all models, except "custom" if \code{matrix} has dimnames.}

\item{afreq}{A numeric vector of allele frequencies. Required in model
"proportional".}

\item{rate}{A number between 0 and 1. Required in models "equal",
"proportional", "stepwise" and "onestep".}

\item{seed}{A single number. Optional parameter in the "random" model, passed
on to \code{set.seed()}.}

\item{rate2}{A number between 0 and 1. The mutation rate between integer
alleles and microvariants. Required in the "stepwise" model.}

\item{range}{A positive number. The relative probability of mutating n+1
steps versus mutating n steps. Required in the "stepwise" and "dawid"
models. Must be in the interval (0,1) for the "dawid" model.}

\item{transform}{Either NULL (default) or one of the strings "MH", "BA",
"PR", "PM". See Details.}

\item{validate}{A logical (default: TRUE) indicating whether to validate
custom models.}

\item{mutmat}{An object of class \code{mutationMatrix}.}
}
\value{
An object of class \code{mutationMatrix}, essentially a square numeric
matrix with various attributes. The matrix has entries in \verb{[0, 1]} and all
rows sum to 1. Both row names and column names are the allele labels.
}
\description{
Construct mutation matrices for pedigree likelihood computations.
}
\details{
Descriptions of the models:
\itemize{
\item \code{custom}: Allows any mutation matrix to be provided by the user, in the
\code{matrix} parameter.
\item \code{dawid}: A reversible model for integer-valued markers, proposed by Dawid
et al. (2002).
\item \code{equal}: All mutations equally likely; probability \eqn{1-rate} of no
mutation.
\item \code{proportional}: Mutation probabilities are proportional to the target
allele frequencies.
\item \code{random}: This produces a matrix of random numbers, where each row is
normalised so that it sums to 1. If \code{rate} (and \code{afreq}) is provided, the
mutation matrix is conditional on the overall mutation rate.
\item \code{onestep}: A mutation model for markers with integer alleles, allowing
mutations only to the nearest neighbours in the allelic ladder. For example,
\code{10} may mutate to either \code{9} or \code{11}, unless \code{10} is the lowest allele, in
which case \code{11} is the only option. This model is not applicable to loci with
non-integer microvariants.
\item \code{stepwise}: A common model in forensic genetics, allowing different
mutation rates between integer alleles (like \code{9}) and non-integer
microvariants (like \code{9.3}). Mutation rates also depend on step size, as
controlled by the \code{range} parameter.
\item \code{trivial}: The identity matrix, implying that no mutations are possible.
}

If \code{transform} is non-NULL, the indicated transformation is applied to the
matrix before returning. Currently, there are 4 available options:
\itemize{
\item \code{MH}, \code{BA}, \code{PR}: See \code{\link[=makeReversible]{makeReversible()}}
\item \code{PM}: See \code{\link[=makeStationary]{makeStationary()}}
}
}
\examples{
mutationMatrix("equal", alleles = 1:3, rate = 0.05)

mutationMatrix("random", afreq = c(a=0.3, b=0.7), rate = 0.05, seed = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getParams.R
\name{getParams}
\alias{getParams}
\title{Get model parameters}
\usage{
getParams(mut, params = NULL, format = 1, sep = "/")
}
\arguments{
\item{mut}{A \code{\link[=mutationModel]{mutationModel()}} or \code{\link[=mutationMatrix]{mutationMatrix()}}.}

\item{params}{A vector contain some or all of the words "model", "rate",
"range", "rate2", "seed". If NULL (default), all present parameters are
included.}

\item{format}{A numeric code indicating the wanted output format. See Value.}

\item{sep}{A separator character used to paste male and female values.
Ignored unless \code{format = 3}.}
}
\value{
When \code{mut} is a \code{mutationModel}, the output format is dictated by the \code{format}
option, with the following possibilities:
\enumerate{
\item A data frame with 2 rows labelled 'female' and 'male'.
\item A data frame with 1 row and female/male columns suffixed by .F/.M
respectively.
\item A data frame with 1 row, in which female/male values are pasted together
(separated with \code{sep}) if different.
\item A list version of format 1, suitable for programmatic use with \code{mutationModel()}.
}

If \code{mut} is a \code{mutationMatrix} the output is always a data frame with 1 row.
}
\description{
Extract model parameters of a mutation matrix/model.
}
\examples{
M = mutationModel("equal", 1:2, rate = list(female = 0.2, male = 0.1))
getParams(M)
getParams(M, format = 2)
getParams(M, format = 3)
getParams(M, format = 3, sep = "|")

pars = getParams(M, format = 4)
pars$alleles = 1:2  # Not part of the parameters, but needed to reconstruct the model.
M2 = do.call(mutationModel, pars)
stopifnot(identical(M, M2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwiseReversible.R
\name{maxRate}
\alias{maxRate}
\title{Upper limits for overall mutation rate for the stepwise reversible model.}
\usage{
maxRate(alleles, afreq, range)
}
\arguments{
\item{alleles}{A character vector with allele labels.}

\item{afreq}{A numeric vector of allele frequencies.}

\item{range}{A positive number.}
}
\value{
A vector of two numbers named \code{UW} and \code{UB}. The first of these is
the maximum overall mutation rate for a well-defined stepwise reversible
mutation matrix with the given input. The latter (UB) is the upper limit of
the overall mutation rate under the additional restraint that the model is
bounded by \code{afreq}.
}
\description{
Upper limits for overall mutation rate for the stepwise reversible model.
}
\author{
Thore Egeland.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lumpMutSpecial.R
\name{lumpMutSpecial}
\alias{lumpMutSpecial}
\title{Special lumping of mutation models}
\usage{
lumpMutSpecial(mut, lump, uSign, afreq = NULL, verbose = TRUE)
}
\arguments{
\item{mut}{A square mutation matrix; typically a \code{\link[=mutationMatrix]{mutationMatrix()}} or
\code{\link[=mutationModel]{mutationModel()}}.}

\item{lump}{A vector containing the alleles to be lumped together.}

\item{uSign}{The U-signature of the pedigree for which lumping is attempted.
See Details.}

\item{afreq}{A vector with allele frequencies, of the same length as the size
of \code{mut}. Extracted from the model if not given.}

\item{verbose}{A logical.}
}
\value{
A reduced mutation model, if lumping was possible, otherwise the
original model is returned unchanged.
}
\description{
This function implements methods for special, or pedigree-aware, allele
lumping. This is typically attempted if the model is not generally lumpable
as determined by \code{\link[=alwaysLumpable]{alwaysLumpable()}}. Note that the resulting lumped model is
tailor-made for a specific likelihood calculation, and may violate the
properties of a well-defined mutation model.
}
\details{
The lumping procedure depends on the location of untyped individuals in the
pedigree, summarised by the so-called U-signature:
\itemize{
\item F-depth: The length of the longest chain of untyped, starting with a founder
\item F-width: The maximum number of children of an untyped founder
\item N-depth: The length of the longest chain of untyped, starting with a nonfounder
\item N-width: The maximum number of children of an untyped nonfounder
}
}
\examples{

af = rep(0.05, 20)
names(af) = 1:20
m = mutationMatrix("random", afreq = af, rate = 0.1, seed = 1)

# Degree 1 lumping
mL = lumpMutSpecial(m, lump = 3:20, uSign = c(1,1,0,0))
mL

# Check
afL = attr(mL, "afreq")
stopifnot(sum(af * m[, 1]) == sum(afL * mL[, 1]))

# Degree 2
mL2 = lumpMutSpecial(m, lump = 3:20, uSign = c(1,2,0,0))
mL2
afL2 = attr(mL2, "afreq")

stopifnot(all.equal(af \%*\% m[, 1]^2, afL2 \%*\% mL2[, 1]^2),
          all.equal(af \%*\% m[, 2]^2, afL2 \%*\% mL2[, 2]^2),
          all.equal(af \%*\% ( m[, 1]*m[, 2]),
                    afL2 \%*\% (mL2[, 1]*mL2[, 2])))

}
\seealso{
\code{\link[=lumpedModel]{lumpedModel()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_properties.R
\name{model_properties}
\alias{model_properties}
\alias{isStationary}
\alias{isReversible}
\alias{isBounded}
\alias{isLumpable}
\alias{alwaysLumpable}
\title{Mutation model properties}
\usage{
isStationary(mutmat, afreq = NULL)

isReversible(mutmat, afreq = NULL)

isBounded(mutmat, afreq = NULL)

isLumpable(mutmat, lump)

alwaysLumpable(mutmat)
}
\arguments{
\item{mutmat}{A \code{\link[=mutationMatrix]{mutationMatrix()}} or a \code{\link[=mutationModel]{mutationModel()}}.}

\item{afreq}{A vector with allele frequencies, of the same length as the size
of \code{mutmat}.}

\item{lump}{A character vector containing a nonempty set of allele labels.}
}
\value{
Each of these functions returns TRUE of FALSE.
}
\description{
Functions that check various properties of a mutation model, including
stationarity, reversibility and lumpability.
}
\details{
The function \code{isBounded()} checks that a mutation model is \emph{bounded} by the
allele frequencies, i.e., that \code{mutmat[i,j] <= afreq[j]} whenever \code{i} is not
equal to \code{j}.

Lumpability is a property of a mutation model that allows aggregating alleles
into groups, or \emph{lumps}, without changing the overall mutation process. The
functions \code{isLumpable()} and \code{alwaysLumpable()} checks lumpability using the
row-sum criterion given by Kemeny & Snell (1976). Note that lumping may be
possible even if the model is not generally lumpable; see \code{\link[=lumpMutSpecial]{lumpMutSpecial()}}
for details.

For each of these functions, if \code{mutmat} is a \code{mutationModel} object, i.e.,
with male and female components, the output is TRUE if and only if both
components satisfy the property in question.
}
\examples{

# "proportional" models are stationary and reversible
afr = c(0.2, 0.3, 0.5)
m_prop = mutationMatrix(model = "prop", alleles = 1:3, afreq = afr, rate = 0.1)
stopifnot(isStationary(m_prop, afr), isReversible(m_prop, afr))

# "equal" model is stationary and reversible only when freqs are equal
m_eq = mutationMatrix(model = "eq", alleles = 1:3, rate = 0.1)
stopifnot(isStationary(m_eq, rep(1/3, 3)), isReversible(m_eq, rep(1/3, 3)))
stopifnot(!isStationary(m_eq, afr), !isReversible(m_eq, afr))

# "equal" and "proportional" models allow allele lumping
stopifnot(isLumpable(m_eq, lump = 1:2))
stopifnot(isLumpable(m_prop, lump = 1:2))

# In fact lumpable for any allele subset
stopifnot(alwaysLumpable(m_eq), alwaysLumpable(m_prop))

}
\references{
Kemeny & Snell (1976). \emph{Finite Markov Chains}. Springer.
}
